package org.hashtree.jbrainhoney.dlap;

import java.lang.reflect.Constructor;
import java.util.List;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Elements;
import org.apache.http.cookie.Cookie;
import org.hashtree.jbrainhoney.dlap.command.*;
import org.hashtree.jbrainhoney.dlap.element.*;
import org.hashtree.jbrainhoney.dlap.enumeration.EntityType;
import org.hashtree.jbrainhoney.dlap.parse.XmlParser;
import org.hashtree.jbrainhoney.network.CommonInternetUri;
import org.hashtree.jbrainhoney.network.http.HttpClient;
import org.hashtree.jbrainhoney.network.http.HttpUri;
import org.hashtree.jbrainhoney.network.http.ThreadSafeHttpClient;
import org.hashtree.jbrainhoney.network.http.request.Body;
import org.hashtree.jbrainhoney.network.http.request.GetRequest;
import org.hashtree.jbrainhoney.network.http.request.PostRequest;
import org.hashtree.jbrainhoney.network.http.request.StringBody;
import org.hashtree.jbrainhoney.webservice.Client;

/**
 * A client for the BrainHoney DLAP API.
 *
 * Additionally, see {@link <a href="http://gls.agilix.com/">the BrainHoney DLAP API guide</a>}. Naming conventions
 * and behaviors are, in large part, kept in sync with this documentation (e.g. command vs. method, element vs.
 * class/entity, command names, element names, etc).
 *
 * Example usage:
 * <pre>{@code DlapClient dlapClient = new DlapClient.Builder().build();}</pre>
 */
public final class DlapClient implements Client {
	/** The default DLAP URI scheme: <i>HttpUri.Scheme.HTTPS</i> */
	public final static HttpUri.Scheme DEFAULT_SCHEME = HttpUri.Scheme.HTTPS;
	/** The default DLAP URI hostname: <i>"gls.agilix.com"</i> */
	public final static String DEFAULT_HOSTNAME = "gls.agilix.com";
	/** The default DLAP URI path: <i>null</i> */
	public final static String DEFAULT_PATH = null;
	/** The default DLAP URI port: <i>null</i> */
	public final static Integer DEFAULT_PORT = null;
	/** The default DLAP URI file: <i>"dlap.ashx"</i> */
	public final static String DEFAULT_FILE = "dlap.ashx";
	/** The default DLAP cookie: <i>"AZT"</i> */
	public final static String DEFAULT_COOKIE = "AZT";

	private final HttpClient httpClient;
	private final HttpUri.Scheme scheme;
	private final String hostname;
	private final String path;
	private final Integer port;
	private final String file;
	private final String cookie;
	private final SessionObserver sessionObserver;

	@SuppressWarnings("synthetic-access")
	private DlapClient(final Builder builder) {
		httpClient = (builder.httpClient != null ? builder.httpClient : new ThreadSafeHttpClient.Builder().build());
		scheme = (builder.scheme != null ? builder.scheme : DEFAULT_SCHEME);
		hostname = (builder.hostname != null ? builder.hostname : DEFAULT_HOSTNAME);
		path = (builder.path != null ? builder.path : DEFAULT_PATH);
		port = (builder.port != null ? builder.port : DEFAULT_PORT);
		file = (builder.file != null ? builder.file : DEFAULT_FILE);
		cookie = (builder.cookie != null ? builder.cookie : DEFAULT_COOKIE);
		sessionObserver = new SessionObserver();
	}

	private static Elements handleXmlResponse(final String response)
		throws IllegalArgumentException, ResponseCodeException, DlapException, SecurityException {

		Element xmlResponse = XmlParser.parseRootElement(response);
		ResponseCode responseCode = null;

		if(xmlResponse == null) throw new IllegalArgumentException("Expected XML response.");
		else if(xmlResponse.getAttribute("code") == null)
			throw new IllegalArgumentException("Expected XML response code attribute.");

		try {
			responseCode = ResponseCode.valueOf(xmlResponse.getAttribute("code").getValue().toUpperCase());
		} catch(IllegalArgumentException exception) {
			responseCode = ResponseCode.EXCEPTION;
		}

		if(responseCode.getExceptionClass() != null) {
			Constructor<? extends ResponseCodeException> exceptionConstructor = null;

			try {
				exceptionConstructor = responseCode.getExceptionClass().getConstructor(String.class);
				throw exceptionConstructor.newInstance(responseCode.getMeaning());
			} catch(java.lang.SecurityException exception) {
				throw exception;
			} catch(java.lang.NoSuchMethodException exception) {
				throw new DlapException(exception);
			} catch(java.lang.IllegalArgumentException exception) {
				throw exception;
			} catch(java.lang.InstantiationException exception) {
				throw new DlapException(exception);
			} catch(java.lang.IllegalAccessException exception) {
				throw new DlapException(exception);
			} catch(java.lang.reflect.InvocationTargetException exception) {
				throw new DlapException(exception);
			}
		}

		return xmlResponse.getChildElements();
	}

	private static String handlePlainTextResponse(final String response) {
		return response;
	}

	private Object execute(final GetRequest request) {
		String response = httpClient.get(request);
		String cleanResponse = response.trim();

		if(cleanResponse.length() > 0) {
			if(cleanResponse.startsWith("<") && cleanResponse.endsWith(">")) return handleXmlResponse(response);
			else return handlePlainTextResponse(response);
		}

		return null;
	}

	private Object execute(final PostRequest request) {
		String response = httpClient.post(request);
		String cleanResponse = response.trim();

		if(cleanResponse.length() > 0) {
			if(cleanResponse.startsWith("<") && cleanResponse.endsWith(">")) return handleXmlResponse(response);
			else return handlePlainTextResponse(response);
		}

		return null;
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers(Commands.newAddGroupMembersCommandBuilder("groupId", enrollmentIds).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> addGroupMembers(final AddGroupMembersCommand command) throws ResponseCodeException {
		Element member = null;
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "addgroupmembers").build())
			.build();
		Element request = new Element("requests");

		for(String enrollmentId : command.getEnrollmentIds()) {
			member = new Element("member");
			member.addAttribute(new Attribute("groupid", command.getGroupId()));
			member.addAttribute(new Attribute("enrollmentid", enrollmentId));
			request.appendChild(member);
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers("groupId", enrollmentIds);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public <T extends Object> List<ResponseCode> addGroupMembers(final String groupId, final List<T> enrollmentIds)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentIds).build());
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers(groupId, enrollmentIds);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public <T extends Object> List<ResponseCode> addGroupMembers(final Integer groupId, final List<T> enrollmentIds)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentIds).build());
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers("groupId", "enrollmentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> addGroupMembers(final String groupId, final String enrollmentId)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers(groupId, "enrollmentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> addGroupMembers(final Integer groupId, final String enrollmentId)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers("groupId", enrollmentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> addGroupMembers(final String groupId, final Integer enrollmentId)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which adds one or more member enrollments to an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.addGroupMembers(groupId, enrollmentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> addGroupMembers(final Integer groupId, final Integer enrollmentId)
		throws ResponseCodeException {

		return addGroupMembers(Commands.newAddGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * Releases DlapClient resources. Be sure to call after you are finished with the DlapClient instance. While a
	 * finalize method exists, relying on it should be considered a bug.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.close();}</pre>
	 */
	@Override
	public void close() {
		httpClient.close();
	}

	/**
	 * A command which copies one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CopyCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCopied = dlapClient.copyCourses(Commands.newCopyCoursesCommandBuilder(courses).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> copyCourses(final CopyCoursesCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "copycourses").build())
			.build();
		Element request = new Element("requests");

		for(Course course : command.getCourses()) {
			request.appendChild(course.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedCourses((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which copies one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CopyCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCopied = dlapClient.copyCourses(courses);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> copyCourses(final List<Course> courses) throws ResponseCodeException {
		return copyCourses(Commands.newCopyCoursesCommandBuilder(courses).build());
	}

	/**
	 * A command which copies one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CopyCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCopied = dlapClient.copyCourses(course);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> copyCourses(final Course course) throws ResponseCodeException {
		return copyCourses(Commands.newCopyCoursesCommandBuilder(course).build());
	}

	/**
	 * A command which creates one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCreated = dlapClient.createCourses(Commands.newCreateCoursesCommandBuilder(courses).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> createCourses(final CreateCoursesCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "createcourses").build())
			.build();
		Element request = new Element("requests");

		for(Course course : command.getCourses()) {
			request.appendChild(course.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedCourses((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which creates one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCreated = dlapClient.createCourses(courses);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> createCourses(final List<Course> courses) throws ResponseCodeException {
		return createCourses(Commands.newCreateCoursesCommandBuilder(courses).build());
	}

	/**
	 * A command which creates one or more courses. Automatically enrolls the calling user as the course owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> coursesCreated = dlapClient.createCourses(course);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> createCourses(final Course course) throws ResponseCodeException {
		return createCourses(Commands.newCreateCoursesCommandBuilder(course).build());
	}

	/**
	 * A command which creates one or more domains and links them as children to the domain specified by parentId. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsCreated = dlapClient.createDomains(Commands.newCreateDomainsCommandBuilder("parentId", domains).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> createDomains(final CreateDomainsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "createdomains").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("parentid", command.getParentId()).build())
			.build();
		Element request = new Element("requests");

		for(Domain domain : command.getDomains()) {
			request.appendChild(domain.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedDomains((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which creates one or more domains and links them as children to the domain specified by parentId. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsCreated = dlapClient.createDomains("parentId", domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> createDomains(final String parentId, final List<Domain> domains) throws ResponseCodeException {
		return createDomains(Commands.newCreateDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which creates one or more domains and links them as children to the domain specified by parentId. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsCreated = dlapClient.createDomains(parentId, domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> createDomains(final Integer parentId, final List<Domain> domains) throws ResponseCodeException {
		return createDomains(Commands.newCreateDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which creates one or more domains and links them as children to the domain specified by parentId. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsCreated = dlapClient.createDomains("parentId", domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> createDomains(final String parentId, final Domain domain) throws ResponseCodeException {
		return createDomains(Commands.newCreateDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which creates one or more domains and links them as children to the domain specified by parentId. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsCreated = dlapClient.createDomains(parentId, domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> createDomains(final Integer parentId, final Domain domain) throws ResponseCodeException {
		return createDomains(Commands.newCreateDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which enrolls a user with the specified rights in a course or section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollmentsCreated = dlapClient.createEnrollments(Commands.newCreateEnrollmentsCommandBuilder(enrollments).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> createEnrollments(final CreateEnrollmentsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "createenrollments").build())
			.build();
		Element request = new Element("requests");

		for(Enrollment enrollment : command.getEnrollments()) {
			request.appendChild(enrollment.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedEnrollments((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which enrolls a user with the specified rights in a course or section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollmentsCreated = dlapClient.createEnrollments(enrollments);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> createEnrollments(final List<Enrollment> enrollments) throws ResponseCodeException {
		return createEnrollments(Commands.newCreateEnrollmentsCommandBuilder(enrollments).build());
	}

	/**
	 * A command which enrolls a user with the specified rights in a course or section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollmentsCreated = dlapClient.createEnrollments(enrollment);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> createEnrollments(final Enrollment enrollment) throws ResponseCodeException {
		return createEnrollments(Commands.newCreateEnrollmentsCommandBuilder(enrollment).build());
	}

	/**
	 * A command which creates one or more groups in the specified owner group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Creategroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groupsCreated = dlapClient.createGroups(Commands.newCreateGroupsCommandBuilder(groups).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> createGroups(final CreateGroupsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "creategroups").build())
			.build();
		Element request = new Element("requests");

		for(Group group : command.getGroups()) {
			request.appendChild(group.toXmlElement());
		}

		body = new StringBody.Builder( new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedGroups((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which creates one or more groups in the specified owner group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Creategroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groupsCreated = dlapClient.createGroups(groups);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> createGroups(final List<Group> groups) throws ResponseCodeException {
		return createGroups(Commands.newCreateGroupsCommandBuilder(groups).build());
	}

	/**
	 * A command which creates one or more groups in the specified owner group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Creategroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groupsCreated = dlapClient.createGroups(group);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> createGroups(final Group group) throws ResponseCodeException {
		return createGroups(Commands.newCreateGroupsCommandBuilder(group).build());
	}

	/**
	 * A command creates one or more sections and enrolls the caller in the section as a section owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sectionsCreated = dlapClient.createSections(Commands.newCreateSectionsCommandBuilder(sections).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> createSections(final CreateSectionsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "createsections").build())
			.build();
		Element request = new Element("requests");

		for(Section section : command.getSections()) {
			request.appendChild(section.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedSections((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command creates one or more sections and enrolls the caller in the section as a section owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sectionsCreated = dlapClient.createSections(sections);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> createSections(final List<Section> sections) throws ResponseCodeException {
		return createSections(Commands.newCreateSectionsCommandBuilder(sections).build());
	}

	/**
	 * A command creates one or more sections and enrolls the caller in the section as a section owner. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sectionsCreated = dlapClient.createSections(section);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> createSections(final Section section) throws ResponseCodeException {
		return createSections(Commands.newCreateSectionsCommandBuilder(section).build());
	}

	/**
	 * A command which creates one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersCreated = dlapClient.createUsers(Commands.newCreateUsersCommandBuilder(users).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> createUsers(final CreateUsersCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "createusers").build())
			.build();
		Element request = new Element("requests");

		for(User user : command.getUsers()) {
			request.appendChild(user.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseNestedUsers((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which creates one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersCreated = dlapClient.createUsers(users);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> createUsers(final List<User> users) throws ResponseCodeException {
		return createUsers(Commands.newCreateUsersCommandBuilder(users).build());
	}

	/**
	 * A command which creates one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersCreated = dlapClient.createUsers(user);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> createUsers(final User user) throws ResponseCodeException {
		return createUsers(Commands.newCreateUsersCommandBuilder(user).build());
	}

	/**
	 * A command which deletes one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteCourses(Commands.newDeleteCoursesCommandBuilder(courses).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteCourses(final DeleteCoursesCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "deletecourses").build())
			.build();
		Element request = new Element("requests");

		for(Course course : command.getCourses()) {
			request.appendChild(course.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which deletes one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteCourses(courses);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteCourses(final List<Course> courses) throws ResponseCodeException {
		return deleteCourses(Commands.newDeleteCoursesCommandBuilder(courses).build());
	}

	/**
	 * A command which deletes one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteCourses(course);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteCourses(final Course course) throws ResponseCodeException {
		return deleteCourses(Commands.newDeleteCoursesCommandBuilder(course).build());
	}

	/**
	 * A command which deletes one or more user enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteEnrollments(Commands.newDeleteEnrollmentsCommandBuilder(enrollments).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteEnrollments(final DeleteEnrollmentsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "deleteenrollments").build())
			.build();
		Element request = new Element("requests");

		for(Enrollment enrollment : command.getEnrollments()) {
			request.appendChild(enrollment.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which deletes one or more user enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteEnrollments(enrollments);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteEnrollments(final List<Enrollment> enrollments) throws ResponseCodeException {
		return deleteEnrollments(Commands.newDeleteEnrollmentsCommandBuilder(enrollments).build());
	}

	/**
	 * A command which deletes one or more user enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteEnrollments(enrollment);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteEnrollments(final Enrollment enrollment) throws ResponseCodeException {
		return deleteEnrollments(Commands.newDeleteEnrollmentsCommandBuilder(enrollment).build());
	}

	/**
	 * A command which deletes one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteGroups(Commands.newDeleteGroupsCommandBuilder(groups).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteGroups(final DeleteGroupsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "deletegroups").build())
			.build();
		Element request = new Element("requests");

		for(Group group : command.getGroups()) {
			request.appendChild(group.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which deletes one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteGroups(groups);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteGroups(final List<Group> groups) throws ResponseCodeException {
		return deleteGroups(Commands.newDeleteGroupsCommandBuilder(groups).build());
	}

	/**
	 * A command which deletes one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteGroups(group);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteGroups(final Group group) throws ResponseCodeException {
		return deleteGroups(Commands.newDeleteGroupsCommandBuilder(group).build());
	}

	/**
	 * A command which deletes one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteSections(Commands.newDeleteSectionsCommandBuilder(sections).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteSections(final DeleteSectionsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "deletesections").build())
			.build();
		Element request = new Element("requests");

		for(Section section : command.getSections()) {
			request.appendChild(section.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which deletes one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteSections(sections);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteSections(final List<Section> sections) throws ResponseCodeException {
		return deleteSections(Commands.newDeleteSectionsCommandBuilder(sections).build());
	}

	/**
	 * A command which deletes one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteSections(section);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteSections(final Section section) throws ResponseCodeException {
		return deleteSections(Commands.newDeleteSectionsCommandBuilder(section).build());
	}

	/**
	 * A command which deletes one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteUsers(Commands.newDeleteUsersCommandBuilder(users).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteUsers(final DeleteUsersCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "deleteusers").build())
			.build();
		Element request = new Element("requests");

		for(User user : command.getUsers()) {
			request.appendChild(user.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which deletes one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteUsers(users);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteUsers(final List<User> users) throws ResponseCodeException {
		return deleteUsers(Commands.newDeleteUsersCommandBuilder(users).build());
	}

	/**
	 * A command which deletes one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/DeleteUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.deleteUsers(user);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> deleteUsers(final User user) throws ResponseCodeException {
		return deleteUsers(Commands.newDeleteUsersCommandBuilder(user).build());
	}

	/**
	 * A command which returns the data posted. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Echo">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code String echo = dlapClient.echo(Commands.newEchoCommandBuilder("Hello World").build());}</pre>
	 *
	 * @return String.
	 */
	public String echo(final EchoCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "echo").build())
			.build();
		Body body = new StringBody.Builder(command.getEcho(), "text/plain").build();

		return (String)execute(new PostRequest.Builder(uri, body).build());
	}

	/**
	 * A command which returns the data posted. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Echo">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code String echo = dlapClient.echo("Hello World");}</pre>
	 *
	 * @return String.
	 */
	public String echo(final String echo) throws ResponseCodeException {
		return echo(Commands.newEchoCommandBuilder(echo).build());
	}

	/** Automatically releases DlapClient resources upon garbage collection. */
	@Override
	protected void finalize() throws Throwable {
		try {
			close();
		} finally {
			super.finalize();
		}
	}

	/**
	 * A command which refreshes the authentication cookie for the current session. The authentication cookie is
	 * refreshed on every call. You can call this command to keep a session alive if the interval between calls is
	 * long (over 10 minutes). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCookie">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.getCookie(Commands.newGetCookieCommandBuilder().build());}</pre>
	 */
	public Void getCookie(final GetCookieCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getcookie").build())
			.build();

		execute(new GetRequest.Builder(uri).build());

		return null;
	}

	/**
	 * A command which refreshes the authentication cookie for the current session. The authentication cookie is
	 * refreshed on every call. You can call this command to keep a session alive if the interval between calls is
	 * long (over 10 minutes). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCookie">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.getCookie(Commands.newGetCookieCommandBuilder().build());}</pre>
	 */
	public Void getCookie() throws ResponseCodeException {
		return getCookie(Commands.newGetCookieCommandBuilder().build());
	}

	/**
	 * A command which gets information for a course. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourse">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Course courseLookup = dlapClient.getCourse(Commands.newGetCourseCommandBuilder("courseId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Course} instance.
	 */
	public Course getCourse(final GetCourseCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getcourse").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("courseid", command.getCourseId()).build())
			.build();

		return XmlParser.parseCourse((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for a course. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourse">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Course courseLookup = dlapClient.getCourse("courseId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Course} instance.
	 */
	public Course getCourse(final String courseId) throws ResponseCodeException {
		return getCourse(Commands.newGetCourseCommandBuilder(courseId).build());
	}

	/**
	 * A command which gets information for a course. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourse">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Course courseLookup = dlapClient.getCourse(courseId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Course} instance.
	 */
	public Course getCourse(final Integer courseId) throws ResponseCodeException {
		return getCourse(Commands.newGetCourseCommandBuilder(courseId).build());
	}

	/**
	 * A command which lists courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourseList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> courses = dlapClient.getCourseList(Commands.newGetCourseListCommandBuilder().build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> getCourseList(final GetCourseListCommand command) throws ResponseCodeException {
		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getcourselist").build());

		if(command.getDomainId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"domainid", command.getDomainId()).build()
			);

		if(command.getReference() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"reference", command.getReference()).build()
			);

		if(command.getTitle() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"title", command.getTitle()).build()
			);

		return XmlParser.parseCourses((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourseList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> courses = dlapClient.getCourseList();}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> getCourseList() throws ResponseCodeException {
		return getCourseList(Commands.newGetCourseListCommandBuilder().build());
	}

	/**
	 * A command which gets information for the specified domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomain">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Domain domain = dlapClient.getDomain(Commands.newGetDomainCommandBuilder("domainId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Domain} instance.
	 */
	public Domain getDomain(final GetDomainCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getdomain").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("domainid", command.getDomainId()).build())
			.build();

		return XmlParser.parseDomain((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for the specified domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomain">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Domain domain = dlapClient.getDomain("domainId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Domain} instance.
	 */
	public Domain getDomain(final String domainId) throws ResponseCodeException {
		return getDomain(Commands.newGetDomainCommandBuilder(domainId).build());
	}

	/**
	 * A command which gets information for the specified domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomain">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Domain domain = dlapClient.getDomain(domainId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Domain} instance.
	 */
	public Domain getDomain(final Integer domainId) throws ResponseCodeException {
		return getDomain(Commands.newGetDomainCommandBuilder(domainId).build());
	}

	/**
	 * A command which lists domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domains = dlapClient.getDomainList(Commands.newGetDomainListCommandBuilder().build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> getDomainList(final GetDomainListCommand command) throws ResponseCodeException {
		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getdomainlist").build());

		if(command.getDomainId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"domainid", command.getDomainId()).build()
			);

		if(command.getName() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"name", command.getName()).build()
			);

		if(command.getUserspace() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"userspace", command.getUserspace()).build()
			);

		return XmlParser.parseDomains((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domains = dlapClient.getDomainList();}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> getDomainList() throws ResponseCodeException {
		return getDomainList(Commands.newGetDomainListCommandBuilder().build());
	}

	/**
	 * A command which gets the list of parent domains for a domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainParentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainParents = dlapClient.getDomainParentList(Commands.newGetDomainParentListCommandBuilder("domainId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> getDomainParentList(final GetDomainParentListCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getdomainparentlist").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("domainid", command.getDomainId()).build())
			.build();

		return XmlParser.parseDomains((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets the list of parent domains for a domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainParentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainParents = dlapClient.getDomainParentList("domainId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> getDomainParentList(final String domainId) throws ResponseCodeException {
		return getDomainParentList(Commands.newGetDomainParentListCommandBuilder(domainId).build());
	}

	/**
	 * A command which gets the list of parent domains for a domain. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainParentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainParents = dlapClient.getDomainParentList(domainId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> getDomainParentList(final Integer domainId) throws ResponseCodeException {
		return getDomainParentList(Commands.newGetDomainParentListCommandBuilder(domainId).build());
	}

	/**
	 * A command which gets information for a enrollment. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollment2">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Enrollment enrollmentLookup = dlapClient.getEnrollment(Commands.newGetEnrollmentCommandBuilder("enrollmentId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instance.
	 */
	public Enrollment getEnrollment(final GetEnrollmentCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getenrollment2").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("enrollmentid", command.getEnrollmentId()).build())
			.build();

		return XmlParser.parseEnrollment((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for a enrollment. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollment2">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Enrollment enrollmentLookup = dlapClient.getEnrollment("enrollmentId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instance.
	 */
	public Enrollment getEnrollment(final String enrollmentId) throws ResponseCodeException {
		return getEnrollment(Commands.newGetEnrollmentCommandBuilder(enrollmentId).build());
	}

	/**
	 * A command which gets information for a enrollment. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollment2">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Enrollment enrollmentLookup = dlapClient.getEnrollment(enrollmentId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instance.
	 */
	public Enrollment getEnrollment(final Integer enrollmentId) throws ResponseCodeException {
		return getEnrollment(Commands.newGetEnrollmentCommandBuilder(enrollmentId).build());
	}

	/**
	 * A command which lists groups that the specified enrollment is a member of. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollmentGroupList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getEnrollmentGroupList(Commands.newGetEnrollmentGroupListCommandBuilder("enrollmentId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getEnrollmentGroupList(final GetEnrollmentGroupListCommand command)
		throws ResponseCodeException {

		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getenrollmentgrouplist").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("enrollmentid", command.getEnrollmentId()).build())
			.build();

		return XmlParser.parseGroups((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which lists groups that the specified enrollment is a member of. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollmentGroupList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getEnrollmentGroupList("enrollmentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getEnrollmentGroupList(final String enrollmentId) throws ResponseCodeException {
		return getEnrollmentGroupList(Commands.newGetEnrollmentGroupListCommandBuilder(enrollmentId).build());
	}

	/**
	 * A command which lists groups that the specified enrollment is a member of. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEnrollmentGroupList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getEnrollmentGroupList(enrollmentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getEnrollmentGroupList(final Integer enrollmentId) throws ResponseCodeException {
		return getEnrollmentGroupList(Commands.newGetEnrollmentGroupListCommandBuilder(enrollmentId).build());
	}

	/**
	 * A command which gets the list of users enrolled in the specified entity. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityEnrollmentList2">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getEntityEnrollmentList(Commands.newGetEntityEnrollmentListCommandBuilder("entityId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getEntityEnrollmentList(final GetEntityEnrollmentListCommand command)
		throws ResponseCodeException {

		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getentityenrollmentlist2").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("entityid", command.getEntityId()).build());

		if(command.getAll() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"allstatus", (command.getAll( ) ? "true" : "false")).build()
			);

		if(command.getFlags() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"flags", command.getFlags().toString()).build()
			);

		if(command.getUserId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"userid", command.getUserId()).build()
			);

		return XmlParser.parseEnrollments((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which gets the list of users enrolled in the specified entity. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityEnrollmentList2">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getEntityEnrollmentList("entityId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getEntityEnrollmentList(final String entityId) throws ResponseCodeException {
		return getEntityEnrollmentList(Commands.newGetEntityEnrollmentListCommandBuilder(entityId).build());
	}

	/**
	 * A command which gets the list of users enrolled in the specified entity. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityEnrollmentList2">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getEntityEnrollmentList(entityId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getEntityEnrollmentList(final Integer entityId) throws ResponseCodeException {
		return getEntityEnrollmentList(Commands.newGetEntityEnrollmentListCommandBuilder(entityId).build());
	}

	/**
	 * A command which gets the entity type (Course, Domain, Group, or Section) for a given entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityType">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code EntityType entityType = dlapClient.getEntityType(Commands.newGetEntityTypeCommandBuilder("entityId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.enumeration.EntityType} instance.
	 */
	public EntityType getEntityType(final GetEntityTypeCommand command)
		throws ResponseCodeException, IllegalArgumentException {

		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getentitytype").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("entityid", command.getEntityId()).build())
			.build();

		return XmlParser.parseEntityType((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets the entity type (Course, Domain, Group, or Section) for a given entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityType">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code EntityType entityType = dlapClient.getEntityType("entityId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.enumeration.EntityType} instance.
	 */
	public EntityType getEntityType(final String entityId) throws ResponseCodeException, IllegalArgumentException {
		return getEntityType(Commands.newGetEntityTypeCommandBuilder(entityId).build());
	}

	/**
	 * A command which gets the entity type (Course, Domain, Group, or Section) for a given entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityType">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code EntityType entityType = dlapClient.getEntityType(entityId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.enumeration.EntityType} instance.
	 */
	public EntityType getEntityType(final Integer entityId) throws ResponseCodeException, IllegalArgumentException {
		return getEntityType(Commands.newGetEntityTypeCommandBuilder(entityId).build());
	}

	/**
	 * A command which gets information for a particular group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroup">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code Group groupLookup = dlapClient.getGroup(Commands.newGetGroupCommandBuilder("groupId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Group} instance.
	 */
	public Group getGroup(final GetGroupCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getgroup").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("groupid", command.getGroupId()).build())
			.build();

		return XmlParser.parseGroup((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for a particular group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroup">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code Group groupLookup = dlapClient.getGroup("groupId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Group} instance.
	 */
	public Group getGroup(final String groupId) throws ResponseCodeException {
		return getGroup(Commands.newGetGroupCommandBuilder(groupId).build());
	}

	/**
	 * A command which gets information for a particular group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroup">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code Group groupLookup = dlapClient.getGroup(groupId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Group} instance.
	 */
	public Group getGroup(final Integer groupId) throws ResponseCodeException {
		return getGroup(Commands.newGetGroupCommandBuilder(groupId).build());
	}

	/**
	 * A command which lists enrollments for the specified group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupEnrollmentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getGroupEnrollmentList(Commands.newGetGroupEnrollmentListCommandBuilder("groupId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getGroupEnrollmentList(final GetGroupEnrollmentListCommand command)
		throws ResponseCodeException {

		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getgroupenrollmentlist").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("groupid", command.getGroupId()).build())
			.build();

		return XmlParser.parseEnrollments((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which lists enrollments for the specified group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupEnrollmentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getGroupEnrollmentList("groupId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getGroupEnrollmentList(final String groupId) throws ResponseCodeException {
		return getGroupEnrollmentList(Commands.newGetGroupEnrollmentListCommandBuilder(groupId).build());
	}

	/**
	 * A command which lists enrollments for the specified group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupEnrollmentList">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getGroupEnrollmentList(groupId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getGroupEnrollmentList(final Integer groupId) throws ResponseCodeException {
		return getGroupEnrollmentList(Commands.newGetGroupEnrollmentListCommandBuilder(groupId).build());
	}

	/**
	 * A command which lists groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getGroupList(Commands.newGetGroupListCommandBuilder("ownerId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getGroupList(final GetGroupListCommand command) throws ResponseCodeException {
		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getgrouplist").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("ownerid", command.getOwnerId()).build());

		if(command.getSetId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"setid", command.getSetId().toString()).build()
			);

		return XmlParser.parseGroups((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getGroupList("ownerId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getGroupList(final String ownerId) throws ResponseCodeException {
		return getGroupList(Commands.newGetGroupListCommandBuilder(ownerId).build());
	}

	/**
	 * A command which lists groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetGroupList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Group> groups = dlapClient.getGroupList(ownerId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Group} instances.
	 */
	public List<Group> getGroupList(final Integer ownerId) throws ResponseCodeException {
		return getGroupList(Commands.newGetGroupListCommandBuilder(ownerId).build());
	}

	/**
	 * A command which gets the rights granted to the specified actor (user or role) for the specified entity (domain,
	 * course, or section). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Right right = dlapClient.getRight(Commands.newGetRightCommandBuilder("actorId", "entityId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Right} instance.
	 */
	public Right getRight(final GetRightCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getrights").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("actorid", command.getActorId()).build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("entityid", command.getEntityId()).build())
			.build();

		return XmlParser.parseRight((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets the rights granted to the specified actor (user or role) for the specified entity (domain,
	 * course, or section). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Right right = dlapClient.getRight("actorId", "entityId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Right} instance.
	 */
	public Right getRight(final String actorId, final String entityId) throws ResponseCodeException {
		return getRight(Commands.newGetRightCommandBuilder(actorId, entityId).build());
	}

	/**
	 * A command which gets the rights granted to the specified actor (user or role) for the specified entity (domain,
	 * course, or section). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Right right = dlapClient.getRight(actorId, "entityId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Right} instance.
	 */
	public Right getRight(final Integer actorId, final String entityId) throws ResponseCodeException {
		return getRight(Commands.newGetRightCommandBuilder(actorId, entityId).build());
	}

	/**
	 * A command which gets the rights granted to the specified actor (user or role) for the specified entity (domain,
	 * course, or section). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Right right = dlapClient.getRight("actorId", entityId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Right} instance.
	 */
	public Right getRight(final String actorId, final Integer entityId) throws ResponseCodeException {
		return getRight(Commands.newGetRightCommandBuilder(actorId, entityId).build());
	}

	/**
	 * A command which gets the rights granted to the specified actor (user or role) for the specified entity (domain,
	 * course, or section). See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Right right = dlapClient.getRight(actorId, entityId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Right} instance.
	 */
	public Right getRight(final Integer actorId, final Integer entityId) throws ResponseCodeException {
		return getRight(Commands.newGetRightCommandBuilder(actorId, entityId).build());
	}

	/**
	 * A command which gets information for a section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSection">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Section sectionLookup = dlapClient.getSection(Commands.newGetSectionCommandBuilder("sectionId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Section} instance.
	 */
	public Section getSection(final GetSectionCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getsection").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("sectionid", command.getSectionId()).build())
			.build();

		return XmlParser.parseSection((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for a section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSection">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Section sectionLookup = dlapClient.getSection("sectionId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Section} instance.
	 */
	public Section getSection(final String sectionId) throws ResponseCodeException {
		return getSection(Commands.newGetSectionCommandBuilder(sectionId).build());
	}

	/**
	 * A command which gets information for a section. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSection">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code Section sectionLookup = dlapClient.getSection(sectionId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.Section} instance.
	 */
	public Section getSection(final Integer sectionId) throws ResponseCodeException {
		return getSection(Commands.newGetSectionCommandBuilder(sectionId).build());
	}

	/**
	 * A command which lists sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSectionList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sections = dlapClient.getSectionList(Commands.newGetSectionListCommandBuilder().build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> getSectionList(final GetSectionListCommand command) throws ResponseCodeException {
		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getsectionlist").build());

		if(command.getCourseId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"courseid", command.getCourseId()).build()
			);

		if(command.getDomainId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"domainid", command.getDomainId()).build()
			);

		if(command.getFlags() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"flags", command.getFlags().toString()).build()
			);

		if(command.getTitle() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"title", command.getTitle()).build()
			);

		return XmlParser.parseSections((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSectionList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sections = dlapClient.getSectionList();}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> getSectionList() throws ResponseCodeException {
		return getSectionList(Commands.newGetSectionListCommandBuilder().build());
	}

	/**
	 * A command which gets information for a particular user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUser">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.getUser(Commands.newGetUserCommandBuilder("userId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User getUser(final GetUserCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getuser").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("userid", command.getUserId()).build())
			.build();

		return XmlParser.parseUser((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which gets information for a particular user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUser">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.getUser("userId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User getUser(final String userId) throws ResponseCodeException {
		return getUser(Commands.newGetUserCommandBuilder(userId).build());
	}

	/**
	 * A command which gets information for a particular user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUser">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.getUser(userId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User getUser(final Integer userId) throws ResponseCodeException {
		return getUser(Commands.newGetUserCommandBuilder(userId).build());
	}

	/**
	 * A command which lists enrollments for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserEnrollmentList2">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getUserEnrollmentList(Commands.newGetUserEnrollmentCommandBuilder("userId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getUserEnrollmentList(final GetUserEnrollmentListCommand command)
		throws ResponseCodeException {

		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getuserenrollmentlist2").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("userid", command.getUserId()).build());

		if(command.getAll() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"allstatus", (command.getAll() ? "true" : "false")).build()
			);

		if(command.getEntityId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"entityid", command.getEntityId()).build()
			);

		if(command.getFlags() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"flags", command.getFlags().toString()).build()
			);

		return XmlParser.parseEnrollments((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists enrollments for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserEnrollmentList2">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getUserEnrollmentList("userId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getUserEnrollmentList(final String userId) throws ResponseCodeException {
		return getUserEnrollmentList(Commands.newGetUserEnrollmentListCommandBuilder(userId).build());
	}

	/**
	 * A command which lists enrollments for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserEnrollmentList2">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.getUserEnrollmentList(userId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> getUserEnrollmentList(final Integer userId) throws ResponseCodeException {
		return getUserEnrollmentList(Commands.newGetUserEnrollmentListCommandBuilder(userId).build());
	}

	/**
	 * A command which lists users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> users = dlapClient.getUserList(Commands.newGetUserListCommandBuilder().build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> getUserList(final GetUserListCommand command) throws ResponseCodeException {
		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "getuserlist").build());

		if(command.getDomainId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"domainid", command.getDomainId()).build()
			);

		if(command.getName() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"name", command.getName()).build()
			);

		if(command.getUsername() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"username", command.getUsername()).build()
			);

		return XmlParser.parseUsers((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserList">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> users = dlapClient.getUserList();}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> getUserList() throws ResponseCodeException {
		return getUserList(Commands.newGetUserListCommandBuilder().build());
	}

	/**
	 * Returns a boolean value as to if there is a current session.
	 *
	 * Example usage:
	 * <pre>{@code boolean hasCookie = dlapClient.hasCookie();}</pre>
	 *
	 * @return boolean.
	 */
	public boolean hasCookie() {
		List<Cookie> cookies = httpClient.getCookies();

		for(Cookie c : cookies) {
			if(c.getName() != null && c.getName().equals(cookie) && c.getValue() != null
				&& c.getValue().trim().length() > 0) return true;
		}

		return false;
	}

	/**
	 * A command which links one or more domains as children to the domain specified by parentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/LinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.linkDomains(Commands.newLinkDomainsCommandBuilder("parentId", domains).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> linkDomains(final LinkDomainsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "linkdomains").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("parentid", command.getParentId()).build())
			.build();
		Element request = new Element("requests");

		for(Domain domain : command.getDomains()) {
			request.appendChild(domain.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which links one or more domains as children to the domain specified by parentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/LinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.linkDomains("parentId", domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> linkDomains(final String parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return linkDomains(Commands.newLinkDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which links one or more domains as children to the domain specified by parentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/LinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.linkDomains(parentId, domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> linkDomains(final Integer parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return linkDomains(Commands.newLinkDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which links one or more domains as children to the domain specified by parentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/LinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.linkDomains("parentId", domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> linkDomains(final String parentId, final Domain domain) throws ResponseCodeException {
		return linkDomains(Commands.newLinkDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which links one or more domains as children to the domain specified by parentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/LinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.linkDomains(parentId, domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> linkDomains(final Integer parentId, final Domain domain) throws ResponseCodeException {
		return linkDomains(Commands.newLinkDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which lists courses that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableCourses">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> courses = dlapClient.listRestorableCourses(Commands.newListRestorableCoursesCommandBuilder("domainId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> listRestorableCourses(final ListRestorableCoursesCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "listrestorablecourses").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("domainid", command.getDomainId()).build())
			.build();

		return XmlParser.parseCourses((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which lists courses that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableCourses">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> courses = dlapClient.listRestorableCourses("domainId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> listRestorableCourses(final String domainId) throws ResponseCodeException {
		return listRestorableCourses(Commands.newListRestorableCoursesCommandBuilder(domainId).build());
	}

	/**
	 * A command which lists courses that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableCourses">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Course> courses = dlapClient.listRestorableCourses(domainId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Course} instances.
	 */
	public List<Course> listRestorableCourses(final Integer domainId) throws ResponseCodeException {
		return listRestorableCourses(Commands.newListRestorableCoursesCommandBuilder(domainId).build());
	}

	/**
	 * A command which lists domains that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableDomains">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsList = dlapClient.listRestorableDomains(Commands.newListRestorableDomainsCommandBuilder("parentId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> listRestorableDomains(final ListRestorableDomainsCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "listrestorabledomains").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("parentid", command.getParentId()).build())
			.build();

		return XmlParser.parseDomains((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which lists domains that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableDomains">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsList = dlapClient.listRestorableDomains("parentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> listRestorableDomains(final String parentId) throws ResponseCodeException {
		return listRestorableDomains(Commands.newListRestorableDomainsCommandBuilder(parentId).build());
	}

	/**
	 * A command which lists domains that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableDomains">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Domain> domainsList = dlapClient.listRestorableDomains(parentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Domain} instances.
	 */
	public List<Domain> listRestorableDomains(final Integer parentId) throws ResponseCodeException {
		return listRestorableDomains(Commands.newListRestorableDomainsCommandBuilder(parentId).build());
	}

	/**
	 * A command which lists enrollments that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableEnrollments">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Enrollment> enrollments = dlapClient.listRestorableEnrollments().setEntityId(Commands.newListRestorableEnrollmentsCommandBuilder("entityId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Enrollment} instances.
	 */
	public List<Enrollment> listRestorableEnrollments(final ListRestorableEnrollmentsCommand command)
		throws ResponseCodeException {

		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "listrestorableenrollments").build());

		if(command.getEntityId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"entityid", command.getEntityId()).build()
			);

		if(command.getUserId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"userid", command.getUserId()).build()
			);

		return XmlParser.parseEnrollments((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists sections that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableSections">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<Section> sections = dlapClient.listRestorableSections(Commands.newListRestorableSectionsCommandBuilder().setDomainId("domainId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.Section} instances.
	 */
	public List<Section> listRestorableSections(final ListRestorableSectionsCommand command)
		throws ResponseCodeException {

		HttpUri.Builder uriBuilder = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "listrestorablesections").build());

		if(command.getCourseId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"courseid", command.getCourseId()).build()
			);

		if(command.getDomainId() != null)
			uriBuilder.addQueryParameter(new CommonInternetUri.QueryParameter.Builder(
				"domainid", command.getDomainId()).build()
			);

		return XmlParser.parseSections((Elements)execute(new GetRequest.Builder(uriBuilder.build()).build()));
	}

	/**
	 * A command which lists users that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersReturned = dlapClient.listRestorableUsers(Commands.newListRestorableUsersCommandBuilder("domainId").build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> listRestorableUsers(final ListRestorableUsersCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "listrestorableusers").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("domainid", command.getDomainId()).build())
			.build();

		return XmlParser.parseUsers((Elements)execute(new GetRequest.Builder(uri).build()));
	}

	/**
	 * A command which lists users that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersReturned = dlapClient.listRestorableUsers("domainId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> listRestorableUsers(final String domainId) throws ResponseCodeException {
		return listRestorableUsers(Commands.newListRestorableUsersCommandBuilder(domainId).build());
	}

	/**
	 * A command which lists users that have been deleted and can be restored. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ListRestorableUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<User> usersReturned = dlapClient.listRestorableUsers(domainId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.element.User} instances.
	 */
	public List<User> listRestorableUsers(final Integer domainId) throws ResponseCodeException {
		return listRestorableUsers(Commands.newListRestorableUsersCommandBuilder(domainId).build());
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Login">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.login(Commands.newLoginCommandBuilder("username", "password").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User login(final LoginCommand command) throws ResponseCodeException, DlapException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file).build();
		Element request = new Element("request");
		command.addObserver(sessionObserver);
		request.addAttribute(new Attribute("cmd", "login"));
		request.addAttribute(
			new Attribute(
				"username",
				(command.getUserspace() != null && command.getUserspace().trim().length() > 0 ? command.getUserspace() + "/" : "") + command.getUsername()
			)
		);
		request.addAttribute(new Attribute("password", command.getPassword()));
		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		try {
			User user = XmlParser.parseUser((Elements)execute(new PostRequest.Builder(uri, body).build()));
			command.getSessionObserverManager().notifyObservers(new SessionEvent(user));

			return user;
		} catch(InvalidCredentialException exception) {
			command.getSessionObserverManager().notifyObservers(new SessionEvent(null));

			throw new DlapException(exception);
		}
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Login">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.login("username", "password");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User login(final String username, final String password) throws ResponseCodeException, DlapException {
		return login(Commands.newLoginCommandBuilder(username, password).addObserver(sessionObserver).build());
	}

	/**
	 * A command which terminates the session state for an authenticated user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Logout">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.logout(Commands.newLogoutCommandBuilder().build());}</pre>
	 */
	public Void logout(final LogoutCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "logout").build())
			.build();
		command.addObserver(sessionObserver);

		execute(new GetRequest.Builder(uri).build());

		command.getSessionObserverManager().notifyObservers(new SessionEvent(null));

		return null;
	}

	/**
	 * A command which terminates the session state for an authenticated user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Logout">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.logout();}</pre>
	 */
	public Void logout() throws ResponseCodeException {
		return logout(Commands.newLogoutCommandBuilder().addObserver(sessionObserver).build());
	}

	/**
	 * A command which starts a session as a different user. The data retrieved from this session appears as if the
	 * proxy user had logged in. The user and proxy user are recorded for any modifications. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Proxy">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.proxy(Commands.newProxyCommandBuilder("userId").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance. Instance will have an
	 * {@link org.hashtree.jbrainhoney.dlap.element.Agent} instance available via the getAgent method.
	 */
	public User proxy(final ProxyCommand command) throws ResponseCodeException, DlapException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "proxy").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("userid", command.getUserId()).build())
			.build();
		command.addObserver(sessionObserver);

		try {
			User user = XmlParser.parseUser((Elements)execute(new GetRequest.Builder(uri).build()));
			command.getSessionObserverManager().notifyObservers(new SessionEvent(user));

			return user;
		} catch(BadRequestException exception) {
			throw new DlapException(exception);
		} catch(InvalidCredentialException exception) {
			throw new DlapException(exception);
		}
	}

	/**
	 * A command which starts a session as a different user. The data retrieved from this session appears as if the
	 * proxy user had logged in. The user and proxy user are recorded for any modifications. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Proxy">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.proxy("userId");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance. Instance will have an
	 * {@link org.hashtree.jbrainhoney.dlap.element.Agent} instance available via the getAgent method.
	 */
	public User proxy(final String userId) throws ResponseCodeException, DlapException {
		return proxy(Commands.newProxyCommandBuilder(userId).addObserver(sessionObserver).build());
	}

	/**
	 * A command which starts a session as a different user. The data retrieved from this session appears as if the
	 * proxy user had logged in. The user and proxy user are recorded for any modifications. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Proxy">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.proxy(userId);}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance. Instance will have an
	 * {@link org.hashtree.jbrainhoney.dlap.element.Agent} instance available via the getAgent method.
	 */
	public User proxy(final Integer userId) throws ResponseCodeException, DlapException {
		return proxy(Commands.newProxyCommandBuilder(userId).addObserver(sessionObserver).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder("groupId", enrollmentIds).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> removeGroupMembers(final RemoveGroupMembersCommand command) throws ResponseCodeException {
		Body body = null;
		Element member = null;
		Element request = new Element("requests");
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "removegroupmembers").build())
			.build();

		for(String enrollmentId : command.getEnrollmentIds()) {
			member = new Element("member");
			member.addAttribute(new Attribute("groupid", command.getGroupId()));
			member.addAttribute(new Attribute("enrollmentid", enrollmentId));

			request.appendChild(member);
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers("groupId", enrollmentIds);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public <T extends Object> List<ResponseCode> removeGroupMembers(final String groupId, final List<T> enrollmentIds)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentIds).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers(groupId, enrollmentIds);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public <T extends Object> List<ResponseCode> removeGroupMembers(final Integer groupId, final List<T> enrollmentIds)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentIds).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers("groupId", "enrollmentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> removeGroupMembers(final String groupId, final String enrollmentId)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers(groupId, "enrollmentId");}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> removeGroupMembers(final Integer groupId, final String enrollmentId)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers("groupId", enrollmentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> removeGroupMembers(final String groupId, final Integer enrollmentId)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which removes one or more member enrollments from an existing group. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RemoveGroupMembers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.removeGroupMembers(groupId, enrollmentId);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> removeGroupMembers(final Integer groupId, final Integer enrollmentId)
		throws ResponseCodeException {

		return removeGroupMembers(Commands.newRemoveGroupMembersCommandBuilder(groupId, enrollmentId).build());
	}

	/**
	 * A command which resets the user password to a random value, and sends the new password to the email address of
	 * the user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ResetPassword">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.resetPassword(Commands.newResetPasswordCommandBuilder("username").build());}</pre>
	 */
	public Void resetPassword(final ResetPasswordCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file).build();
		Element request = new Element("request");
		request.addAttribute(new Attribute("cmd", "resetpassword"));
		request.addAttribute(
			new Attribute(
				"username",
				(command.getUserspace() != null && command.getUserspace().trim().length() > 0 ? command.getUserspace() + "/" : "") + command.getUsername()
			)
		);

		if(command.getAnswer() != null)
			request.addAttribute(new Attribute("answer", command.getAnswer()));

		if(command.getBody() != null)
			request.addAttribute(new Attribute("body", command.getBody()));

		if(command.getDisplay() != null)
			request.addAttribute(new Attribute("display", command.getDisplay()));

		if(command.getFrom() != null)
			request.addAttribute(new Attribute("from", command.getFrom()));

		if(command.getSubject() != null)
			request.addAttribute(new Attribute("subject", command.getSubject()));

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		execute(new PostRequest.Builder(uri, body).build());

		return null;
	}

	/**
	 * A command which resets the user password to a random value, and sends the new password to the email address
	 * of the user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/ResetPassword">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.resetPassword("username");}</pre>
	 */
	public Void resetPassword(final String username) throws ResponseCodeException {
		return resetPassword(Commands.newResetPasswordCommandBuilder(username).build());
	}

	/**
	 * A command which restores one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreCourses(Commands.newRestoreCoursesCommandBuilder(courses).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreCourses(final RestoreCoursesCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "restorecourses").build())
			.build();
		Element request = new Element("requests");

		for(Course course : command.getCourses()) {
			request.appendChild(course.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which restores one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreCourses(courses);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreCourses(final List<Course> courses) throws ResponseCodeException {
		return restoreCourses(Commands.newRestoreCoursesCommandBuilder(courses).build());
	}

	/**
	 * A command which restores one or more courses. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreCourses(course);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreCourses(final Course course) throws ResponseCodeException {
		return restoreCourses(Commands.newRestoreCoursesCommandBuilder(course).build());
	}

	/**
	 * A command which restores one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreDomains(Commands.newRestoreDomainsCommandBuilder("parentId", domains).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreDomains(final RestoreDomainsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "restoredomains").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("parentid", command.getParentId()).build())
			.build();
		Element request = new Element("requests");

		for(Domain domain : command.getDomains()) {
			request.appendChild(domain.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which restores one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreDomains("parentId", domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreDomains(final String parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return restoreDomains(Commands.newRestoreDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which restores one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreDomains(parentId, domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreDomains(final Integer parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return restoreDomains(Commands.newRestoreDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which restores one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreDomains("parentId", domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreDomains(final String parentId, final Domain domain) throws ResponseCodeException {
		return restoreDomains(Commands.newRestoreDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which restores one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreDomains(parentId, domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreDomains(final Integer parentId, final Domain domain) throws ResponseCodeException {
		return restoreDomains(Commands.newRestoreDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which restores one or more enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreEnrollments">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreEnrollments(Commands.newRestoreEnrollmentsCommandBuilder(enrollments).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreEnrollments(final RestoreEnrollmentsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "restoreenrollments").build())
			.build();
		Element request = new Element("requests");

		for(Enrollment enrollment : command.getEnrollments()) {
			request.appendChild(enrollment.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which restores one or more enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreEnrollments">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreEnrollments(enrollments);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreEnrollments(final List<Enrollment> enrollments) throws ResponseCodeException {
		return restoreEnrollments(Commands.newRestoreEnrollmentsCommandBuilder(enrollments).build());
	}

	/**
	 * A command which restores one or more enrollments. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreEnrollments">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreEnrollments(enrollment);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreEnrollments(final Enrollment enrollment) throws ResponseCodeException {
		return restoreEnrollments(Commands.newRestoreEnrollmentsCommandBuilder(enrollment).build());
	}

	/**
	 * A command which restores one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreSections(Commands.newRestoreSectionsCommandBuilder(sections).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreSections(final RestoreSectionsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "restoresections").build())
			.build();
		Element request = new Element("requests");

		for(Section section : command.getSections()) {
			request.appendChild(section.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which restores one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreSections(sections);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreSections(final List<Section> sections) throws ResponseCodeException {
		return restoreSections(Commands.newRestoreSectionsCommandBuilder(sections).build());
	}

	/**
	 * A command which restores one or more sections. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreSections(section);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreSections(final Section section) throws ResponseCodeException {
		return restoreSections(Commands.newRestoreSectionsCommandBuilder(section).build());
	}

	/**
	 * A command which restores one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreUsers(Commands.newRestoreUsersCommandBuilder(users).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreUsers(final RestoreUsersCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "restoreusers").build())
			.build();
		Element request = new Element("requests");

		for(User user : command.getUsers()) {
			request.appendChild(user.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which restores one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreUsers(users);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreUsers(final List<User> users) throws ResponseCodeException {
		return restoreUsers(Commands.newRestoreUsersCommandBuilder(users).build());
	}

	/**
	 * A command which restores one or more users. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/RestoreUsers">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.restoreUsers(user);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> restoreUsers(final User user) throws ResponseCodeException {
		return restoreUsers(Commands.newRestoreUsersCommandBuilder(user).build());
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state using a SAML assertion for
	 * single sign-on. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SamlLogin">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.samlLogin(Commands.newSamlLoginCommandBuilder("userspace", "response").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User samlLogin(final SamlLoginCommand command) throws ResponseCodeException, DlapException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file).build();
		Element request = new Element("request");
		command.addObserver(sessionObserver);
		request.addAttribute(new Attribute("cmd", "login"));
		request.addAttribute(new Attribute("userspace", command.getUserspace()));
		request.addAttribute(new Attribute("samlresponse", command.getResponse()));
		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		try {
			User user = XmlParser.parseUser((Elements)execute(new PostRequest.Builder(uri, body).build()));
			command.getSessionObserverManager().notifyObservers(new SessionEvent(user));

			return user;
		} catch(InvalidCredentialException exception) {
			command.getSessionObserverManager().notifyObservers(new SessionEvent(null));

			throw new DlapException(exception);
		}
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state using a SAML assertion for
	 * single sign-on. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SamlLogin">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.samlLogin("userspace", "response");}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User samlLogin(final String userspace, final String response) throws ResponseCodeException, DlapException {
		return samlLogin(Commands.newSamlLoginCommandBuilder(userspace, response).addObserver(sessionObserver).build());
	}

	/**
	 * A command which sends an e-mail to recipients that are enrolled in the same entity as that specified by the
	 * sender's enrollmentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SendMail">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.sendEmail(Commands.newSendEmailCommandBuilder("enrollmentId", email).build());}</pre>
	 */
	public Void sendEmail(final SendEmailCommand command) throws ResponseCodeException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "sendmail").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("enrollmentid", command.getEnrollmentId()).build())
			.build();
		Body body = new StringBody.Builder(
			new Document(command.getEmail().toXmlElement()).toXML(), "application/x-dlap-mail-xml"
		).build();

		XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));

		return null;
	}

	/**
	 * A command which sends an e-mail to recipients that are enrolled in the same entity as that specified by the
	 * sender's enrollmentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SendMail">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.sendEmail("enrollmentId", email);}</pre>
	 */
	public Void sendEmail(final String enrollmentId, final Email email) throws ResponseCodeException {
		return sendEmail(Commands.newSendEmailCommandBuilder(enrollmentId, email).build());
	}

	/**
	 * A command which sends an e-mail to recipients that are enrolled in the same entity as that specified by the
	 * sender's enrollmentId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SendMail">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.sendEmail(enrollmentId, email);}</pre>
	 */
	public Void sendEmail(final Integer enrollmentId, final Email email) throws ResponseCodeException {
		return sendEmail(Commands.newSendEmailCommandBuilder(enrollmentId, email).build());
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state using a SSO provider. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SSOLogin">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.ssoLogin(Commands.newSsoLoginCommandBuilder("system", "domainId", "ticket", "service").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User ssoLogin(final SsoLoginCommand command) throws ResponseCodeException, DlapException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "ssologin").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("system", command.getSystem()).build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("domainid", command.getDomainId()).build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("ticket", command.getTicket()).build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("service", command.getService()).build())
			.build();
		command.addObserver(sessionObserver);

		try {
			User user = XmlParser.parseUser((Elements)execute(new GetRequest.Builder(uri).build()));
			command.getSessionObserverManager().notifyObservers(new SessionEvent(user));

			return user;
		} catch(InvalidCredentialException exception) {
			command.getSessionObserverManager().notifyObservers(new SessionEvent(null));

			throw new CommandException(exception);
		}
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state using a SSO provider. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SSOLogin">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.ssoLogin("system", "domainId", "ticket", "service").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User ssoLogin(final String system, final String domainId, final String ticket, final String service)
		throws ResponseCodeException, DlapException {

		return ssoLogin(
			Commands.newSsoLoginCommandBuilder(system, domainId, ticket, service).addObserver(sessionObserver).build()
		);
	}

	/**
	 * A command which authenticates a user on the server and initiates a session state using a SSO provider. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/SSOLogin">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.ssoLogin(("system", domainId, "ticket", "service").build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User ssoLogin(final String system, final Integer domainId, final String ticket, final String service)
		throws ResponseCodeException, DlapException {

		return ssoLogin(Commands.newSsoLoginCommandBuilder(system, domainId, ticket, service).addObserver(sessionObserver).build());
	}

	/**
	 * A command which unlinks one or more domains from the domain specified by parentId and it deletes the domain if no
	 * other links exist. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UnlinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.unlinkDomains(Commands.newUnlinkDomainsCommandBuilder("parentId", domains).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> unlinkDomains(final UnlinkDomainsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "unlinkdomains").build())
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("parentid", command.getParentId()).build())
			.build();
		Element request = new Element("requests");

		for(Domain domain : command.getDomains()) {
			request.appendChild(domain.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which unlinks one or more domains from the domain specified by parentId and it deletes the domain if no
	 * other links exist. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UnlinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.unlinkDomains("parentId", domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> unlinkDomains(final String parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return unlinkDomains(Commands.newUnlinkDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which unlinks one or more domains from the domain specified by parentId and it deletes the domain if no
	 * other links exist. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UnlinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.unlinkDomains(parentId, domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> unlinkDomains(final Integer parentId, final List<Domain> domains)
		throws ResponseCodeException {

		return unlinkDomains(Commands.newUnlinkDomainsCommandBuilder(parentId, domains).build());
	}

	/**
	 * A command which unlinks one or more domains from the domain specified by parentId and it deletes the domain if no
	 * other links exist. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UnlinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.unlinkDomains("parentId", domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> unlinkDomains(final String parentId, final Domain domain) throws ResponseCodeException {
		return unlinkDomains(Commands.newUnlinkDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which unlinks one or more domains from the domain specified by parentId and it deletes the domain if no
	 * other links exist. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UnlinkDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.unlinkDomains(parentId, domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> unlinkDomains(final Integer parentId, final Domain domain) throws ResponseCodeException {
		return unlinkDomains(Commands.newUnlinkDomainsCommandBuilder(parentId, domain).build());
	}

	/**
	 * A command which ends a proxy session. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Unproxy">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.unproxy(Commands.newUnproxyCommandBuilder().build());}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User unproxy(final UnproxyCommand command) throws ResponseCodeException, DlapException {
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "unproxy").build())
			.build();
		command.addObserver(sessionObserver);

		try {
			User user = XmlParser.parseUser((Elements)execute(new GetRequest.Builder(uri).build()));
			command.getSessionObserverManager().notifyObservers(new SessionEvent(user));

			return user;
		} catch(InvalidOperationException exception) {
			throw new DlapException(exception);
		}
	}

	/**
	 * A command which ends a proxy session. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/Unproxy">the BrainHoney DLAP API guide</a>} for this command.
	 *
	 * Example usage:
	 * <pre>{@code User user = dlapClient.unproxy();}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance.
	 */
	public User unproxy() throws ResponseCodeException, DlapException {
		return unproxy(Commands.newUnproxyCommandBuilder().addObserver(sessionObserver).build());
	}

	/**
	 * A command which updates the updates the title, reference, data, and other attributes of one or more courses. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateCourses(Commands.newUpdateCoursesCommandBuilder(courses).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateCourses(final UpdateCoursesCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updatecourses").build())
			.build();
		Element request = new Element("requests");

		for(Course course : command.getCourses()) {
			request.appendChild(course.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the updates the title, reference, data, and other attributes of one or more courses. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateCourses(courses);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateCourses(final List<Course> courses) throws ResponseCodeException {
		return updateCourses(Commands.newUpdateCoursesCommandBuilder(courses).build());
	}

	/**
	 * A command which updates the updates the title, reference, data, and other attributes of one or more courses. See
	 * the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateCourses">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateCourses(course);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateCourses(final Course course) throws ResponseCodeException {
		return updateCourses(Commands.newUpdateCoursesCommandBuilder(course).build());
	}

	/**
	 * A command which updates the name and reference value of one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateDomains(Commands.newUpdateDomainsCommandBuilder(domains).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateDomains(final UpdateDomainsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updatedomains").build())
			.build();
		Element request = new Element("requests");

		for(Domain domain : command.getDomains()) {
			request.appendChild(domain.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the name and reference value of one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateDomains(domains);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateDomains(final List<Domain> domains) throws ResponseCodeException {
		return updateDomains(Commands.newUpdateDomainsCommandBuilder(domains).build());
	}

	/**
	 * A command which updates the name and reference value of one or more domains. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateDomains">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateDomains(domain);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateDomains(final Domain domain) throws ResponseCodeException {
		return updateDomains(Commands.newUpdateDomainsCommandBuilder(domain).build());
	}

	/**
	 * A command which updates the enrollment status, startDate, endDate, and rights granted to the specified user for a
	 * course or section. You can also change an enrollment from one section to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateEnrollments(Commands.newUpdateEnrollmentsCommandBuilder(enrollments).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateEnrollments(final UpdateEnrollmentsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updateenrollments").build())
			.build();
		Element request = new Element("requests");

		for(Enrollment enrollment : command.getEnrollments()) {
			request.appendChild(enrollment.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the enrollment status, startDate, endDate, and rights granted to the specified user for a
	 * course or section. You can also change an enrollment from one section to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateEnrollments(enrollments);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateEnrollments(final List<Enrollment> enrollments) throws ResponseCodeException {
		return updateEnrollments(Commands.newUpdateEnrollmentsCommandBuilder(enrollments).build());
	}

	/**
	 * A command which updates the enrollment status, startDate, endDate, and rights granted to the specified user for a
	 * course or section. You can also change an enrollment from one section to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateEnrollments">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateEnrollments(enrollment);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateEnrollments(final Enrollment enrollment) throws ResponseCodeException {
		return updateEnrollments(Commands.newUpdateEnrollmentsCommandBuilder(enrollment).build());
	}

	/**
	 * A command which updates the title, setId, reference, and data of one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateGroups(Commands.newUpdateGroupsCommandBuilder(groups).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateGroups(final UpdateGroupsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updategroups").build())
			.build();
		Element request = new Element("requests");

		for(Group group : command.getGroups()) {
			request.appendChild(group.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the title, setId, reference, and data of one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateGroups(groups);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateGroups(final List<Group> groups) throws ResponseCodeException {
		return updateGroups(Commands.newUpdateGroupsCommandBuilder(groups).build());
	}

	/**
	 * A command which updates the title, setId, reference, and data of one or more groups. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateGroups">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateGroups(group);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateGroups(final Group group) throws ResponseCodeException {
		return updateGroups(Commands.newUpdateGroupsCommandBuilder(group).build());
	}

	/**
	 * A command which updates the password of the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePassword">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePassword(Commands.newUpdatePasswordCommandBuilder("userId", "password").build());}</pre>
	 */
	public Void updatePassword(final UpdatePasswordCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file).build();
		Element request = new Element("request");
		request.addAttribute(new Attribute("cmd", "updatepassword"));
		request.addAttribute(new Attribute("userid", command.getUserId()));
		request.addAttribute(new Attribute("password", command.getPassword()));

		if(command.getOldPassword() != null)
			request.addAttribute(new Attribute("oldpassword", command.getOldPassword()));

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		execute(new PostRequest.Builder(uri, body).build());

		return null;
	}

	/**
	 * A command which updates the password of the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePassword">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePassword("userId", "password");}</pre>
	 */
	public Void updatePassword(final String userId, final String password) throws ResponseCodeException {
		return updatePassword(Commands.newUpdatePasswordCommandBuilder(userId, password).build());
	}

	/**
	 * A command which updates the password of the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePassword">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePassword(userId, "password");}</pre>
	 */
	public Void updatePassword(final Integer userId, final String password) throws ResponseCodeException {
		return updatePassword(Commands.newUpdatePasswordCommandBuilder(userId, password).build());
	}

	/**
	 * A command which updates the password question and answer for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePasswordQuestionAnswer">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePasswordQuestionAnswer(Commands.newUpdatePasswordQuestionAnswerCommandBuilder("userId", "question", "answer").build());}</pre>
	 */
	public Void updatePasswordQuestionAnswer(final UpdatePasswordQuestionAnswerCommand command)
		throws ResponseCodeException {

		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file).build();
		Element request = new Element("request");
		request.addAttribute(new Attribute("cmd", "updatepasswordquestionanswer"));
		request.addAttribute(new Attribute("userid", command.getUserId()));
		request.addAttribute(new Attribute("passwordquestion", command.getPasswordQuestion()));
		request.addAttribute(new Attribute("passwordanswer", command.getPasswordAnswer()));

		if(command.getOldPassword() != null)
			request.addAttribute(new Attribute("oldpassword", command.getOldPassword()));

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		execute(new PostRequest.Builder(uri, body).build());

		return null;
	}

	/**
	 * A command which updates the password question and answer for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePasswordQuestionAnswer">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePasswordQuestionAnswer("userId", "question", "answer");}</pre>
	 */
	public Void updatePasswordQuestionAnswer(final String userId, final String passwordQuestion, final String passwordAnswer)
		throws ResponseCodeException {

		return updatePasswordQuestionAnswer(
			Commands.newUpdatePasswordQuestionAnswerCommandBuilder(userId, passwordQuestion, passwordAnswer).build()
		);
	}

	/**
	 * A command which updates the password question and answer for the specified user. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePasswordQuestionAnswer">the BrainHoney DLAP API guide</a>}
	 * for this command.
	 *
	 * Example usage:
	 * <pre>{@code dlapClient.updatePasswordQuestionAnswer(userId, "question", "answer");}</pre>
	 */
	public Void updatePasswordQuestionAnswer(final Integer userId, final String passwordQuestion, final String passwordAnswer)
		throws ResponseCodeException {

		return updatePasswordQuestionAnswer(
			Commands.newUpdatePasswordQuestionAnswerCommandBuilder(userId, passwordQuestion, passwordAnswer).build()
		);
	}

	/**
	 * A command which updates the rights (flags) granted to the specified user (actorId) for the domain specified by
	 * entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateRights(Commands.newUpdateRightsCommandBuilder(rights).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateRights(final UpdateRightsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updaterights").build())
			.build();
		Element request = new Element("requests");

		for(Right right : command.getRights()) {
			request.appendChild(right.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the rights (flags) granted to the specified user (actorId) for the domain specified by
	 * entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateRights(rights);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateRights(final List<Right> rights) throws ResponseCodeException {
		return updateRights(Commands.newUpdateRightsCommandBuilder(rights).build());
	}

	/**
	 * A command which updates the rights (flags) granted to the specified user (actorId) for the domain specified by
	 * entityId. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateRights">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateRights(right);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateRights(final Right right) throws ResponseCodeException {
		return updateRights(Commands.newUpdateRightsCommandBuilder(right).build());
	}

	/**
	 * A command which updates the title, reference, type, startDate, endDate, and days attributes of one or more
	 * sections. It updates only the specified attributes. Any omitted attributes retain their existing values. You can
	 * also change a section from one course to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateSections(Commands.newUpdateSectionsCommandBuilder(sections).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateSections(final UpdateSectionsCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updatesections").build())
			.build();
		Element request = new Element("requests");

		for(Section section : command.getSections()) {
			request.appendChild(section.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the title, reference, type, startDate, endDate, and days attributes of one or more
	 * sections. It updates only the specified attributes. Any omitted attributes retain their existing values. You can
	 * also change a section from one course to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateSections(sections);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateSections(final List<Section> sections) throws ResponseCodeException {
		return updateSections(Commands.newUpdateSectionsCommandBuilder(sections).build());
	}

	/**
	 * A command which updates the title, reference, type, startDate, endDate, and days attributes of one or more
	 * sections. It updates only the specified attributes. Any omitted attributes retain their existing values. You can
	 * also change a section from one course to another. See the
	 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateSections">the BrainHoney DLAP API guide</a>} for this
	 * command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateSections(section);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateSections(final Section section) throws ResponseCodeException {
		return updateSections(Commands.newUpdateSectionsCommandBuilder(section).build());
	}

	/**
	 * A command which updates the firstName, lastName, reference, or email for one or more users specified by userId.
	 * See the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateUsers(Commands.newUpdateUsersCommandBuilder(users).build());}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateUsers(final UpdateUsersCommand command) throws ResponseCodeException {
		Body body = null;
		HttpUri uri = new HttpUri.Builder(scheme, hostname).setPort(port).setPath(path).setFile(file)
			.addQueryParameter(new CommonInternetUri.QueryParameter.Builder("cmd", "updateusers").build())
			.build();
		Element request = new Element("requests");

		for(User user : command.getUsers()) {
			request.appendChild(user.toXmlElement());
		}

		body = new StringBody.Builder(new Document(request).toXML(), "text/xml").build();

		return XmlParser.parseResponseCodes((Elements)execute(new PostRequest.Builder(uri, body).build()));
	}

	/**
	 * A command which updates the firstName, lastName, reference, or email for one or more users specified by userId.
	 * See the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateUsers(users);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateUsers(final List<User> users) throws ResponseCodeException {
		return updateUsers(Commands.newUpdateUsersCommandBuilder(users).build());
	}

	/**
	 * A command which updates the firstName, lastName, reference, or email for one or more users specified by userId.
	 * See the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateUsers">the BrainHoney DLAP API guide</a>} for
	 * this command.
	 *
	 * Example usage:
	 * <pre>{@code List<ResponseCode> responseCodes = dlapClient.updateUsers(user);}</pre>
	 *
	 * @return A list of {@link org.hashtree.jbrainhoney.dlap.ResponseCode} instances.
	 */
	public List<ResponseCode> updateUsers(final User user) throws ResponseCodeException {
		return updateUsers(Commands.newUpdateUsersCommandBuilder(user).build());
	}

	/**
	 * Returns the user of last login or proxy.
	 *
	 * Example usage:
	 * <pre>{@code User user dlapClient.whoAmI();}</pre>
	 *
	 * @return A {@link org.hashtree.jbrainhoney.dlap.element.User} instance. Instance will have an
	 * {@link org.hashtree.jbrainhoney.dlap.element.Agent} instance available via the getAgent method if it is a proxy.
	 */
	public User whoAmI() {
		return sessionObserver.getUser();
	}

	/** A builder of immutable DlapClient instances. */
	public final static class Builder {
		private HttpClient httpClient;
		private HttpUri.Scheme scheme;
		private String hostname;
		private String path;
		private Integer port;
		private String file;
		private String cookie;

		public Builder() {

		}

		public Builder setHttpClient(final HttpClient httpClient) {
			this.httpClient = httpClient;

			return this;
		}

		public Builder setScheme(final HttpUri.Scheme scheme) {
			this.scheme = scheme;

			return this;
		}

		public Builder setHostname(final String hostname) {
			this.hostname = hostname;

			return this;
		}

		public Builder setPath(final String path) {
			this.path = path;

			return this;
		}

		public Builder setPort(final Integer port) {
			this.port = port;

			return this;
		}

		public Builder setFile(final String file) {
			this.file = file;

			return this;
		}

		public Builder setCookie(final String cookie) {
			this.cookie = cookie;

			return this;
		}

		@SuppressWarnings("synthetic-access")
		public DlapClient build() {
			return new DlapClient(this);
		}
	}
}