package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which starts a session as a different user. The data retrieved from this session appears as if the proxy
 * user had logged in. The user and proxy user are recorded for any modifications. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/Proxy">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class ProxyCommand implements Command, ObservableSessionCommand {
	private final SessionObserverManager sessionObserverManager;
	private final String userId;

	@SuppressWarnings("synthetic-access")
	private ProxyCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.sessionObserverManager == null)
			throw new IllegalArgumentException("Expected sessionObserverManager to not be null.");
		else if(builder.userId == null) throw new IllegalArgumentException("Expected userId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.userId))
			throw new IllegalArgumentException("Expected userId to be valid.");

		sessionObserverManager = builder.sessionObserverManager;
		userId = builder.userId;
	}

	public SessionObserverManager getSessionObserverManager() {
		return sessionObserverManager;
	}

	public String getUserId() {
		return userId;
	}

	@Override
	public ProxyCommand addObserver(final SessionObserver observer) {
		sessionObserverManager.addObserver(observer);

		return this;
	}

	@Override
	public ProxyCommand removeObserver(final SessionObserver observer) {
		sessionObserverManager.removeObserver(observer);

		return this;
	}

	/** A builder of immutable ProxyCommand instances. */
	public final static class Builder implements Command.Builder, ObservableSessionCommand.Builder {
		private final SessionObserverManager sessionObserverManager;
		private final String userId;

		public Builder(final String userId) {
			super();

			this.sessionObserverManager = new SessionObserverManager();
			this.userId = userId;
		}

		public Builder(final Integer userId) {
			super();

			this.sessionObserverManager = new SessionObserverManager();
			this.userId = (userId == null ? null : userId.toString());
		}

		@Override
		public Builder addObserver(final SessionObserver observer) {
			sessionObserverManager.addObserver(observer);

			return this;
		}

		@Override
		public Builder removeObserver(final SessionObserver observer) {
			sessionObserverManager.removeObserver(observer);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public ProxyCommand build() {
			return new ProxyCommand(this);
		}
	}
}