package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.UsernameAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.UserspaceAttributeValidator;

/**
 * A command which resets the user password to a random value, and sends the new password to the email address of the
 * user. See the {@link <a href="http://gls.agilix.com/Docs/Command/ResetPassword">the BrainHoney DLAP API guide</a>}
 * for this command.
 */
public final class ResetPasswordCommand implements Command {
	private final String answer;
	private final String body;
	private final String display;
	private final String from;
	private final String subject;
	private final String username;
	private final String userspace;

	@SuppressWarnings("synthetic-access")
	private ResetPasswordCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.body != null && builder.body.indexOf("{0}") == -1)
			throw new IllegalArgumentException("Expected body to contain password variable.");
		else if(builder.username == null) throw new IllegalArgumentException("Expected username to not be null.");
		else if(!UsernameAttributeValidator.getInstance().isValid(builder.username))
			throw new IllegalArgumentException("Expected username to be valid.");
		else if(builder.userspace != null && !UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");

		answer = builder.answer;
		body = builder.body;
		display = builder.display;
		from = builder.from;
		subject = builder.subject;
		username = builder.username;
		userspace = builder.userspace;
	}

	public String getAnswer() {
		return answer;
	}

	public String getBody() {
		return body;
	}

	public String getDisplay() {
		return display;
	}

	public String getFrom() {
		return from;
	}

	public String getSubject() {
		return subject;
	}

	public String getUsername() {
		return username;
	}

	public String getUserspace() {
		return userspace;
	}

	/** A builder of immutable ResetPasswordCommand instances. */
	public final static class Builder implements Command.Builder {
		private String answer;
		private String body;
		private String display;
		private String from;
		private String subject;
		private final String username;
		private String userspace;

		public Builder(final String username) {
			super();

			this.username = username;
		}

		public Builder setAnswer(final String answer) {
			this.answer = answer;

			return this;
		}

		public Builder setBody(final String body) {
			this.body = body;

			return this;
		}

		public Builder setDisplay(final String display) {
			this.display = display;

			return this;
		}

		public Builder setFrom(final String from) {
			this.from = from;

			return this;
		}

		public Builder setSubject(final String subject) {
			this.subject = subject;

			return this;
		}

		public Builder setUserspace(final String userspace) {
			this.userspace = userspace;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public ResetPasswordCommand build() {
			return new ResetPasswordCommand(this);
		}
	}
}