package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Course;

/**
 * A command which updates the updates the title, reference, data, and other attributes of one or more courses. See
 * the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateCourses">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class UpdateCoursesCommand implements Command {
	private final List<Course> courses;

	@SuppressWarnings("synthetic-access")
	private UpdateCoursesCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.courses == null || builder.courses.size() == 0)
			throw new IllegalArgumentException("Expected courses to not be null.");

		for(Course course : builder.courses) {
			if(course == null) throw new IllegalArgumentException("Expected course to not be null.");
			else if(course.getCourseId() == null || (course.getData() == null && course.getDays() == null
				&& course.getEndDate() == null && course.getReference() == null && course.getStartDate() == null
				&& course.getTerm() == null && course.getTitle() == null && course.getType() == null))

				throw new IllegalArgumentException("Expected courseId and data, days, endDate, reference, startDate, term, title, and type to not be null.");
		}

		courses = Collections.unmodifiableList(new ArrayList<Course>(builder.courses));
	}

	public List<Course> getCourses() {
		return courses;
	}

	/** A builder of immutable UpdateCoursesCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Course> courses;

		public Builder(final List<Course> courses) {
			super();

			if(courses == null) this.courses = Collections.synchronizedList(new ArrayList<Course>());
			else this.courses = Collections.synchronizedList(new ArrayList<Course>(courses));
		}

		public Builder(final Course course) {
			super();

			if(course == null) courses = Collections.synchronizedList(new ArrayList<Course>());
			else courses = Collections.synchronizedList(new ArrayList<Course>(Arrays.asList(course)));
		}

		public Builder addCourse(final Course course) {
			courses.add(course);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdateCoursesCommand build() {
			return new UpdateCoursesCommand(this);
		}
	}
}