package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Enrollment;

/**
 * A command which updates the enrollment status, startDate, endDate, and rights granted to the specified user for a
 * course or section. You can also change an enrollment from one section to another. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateEnrollments">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class UpdateEnrollmentsCommand implements Command {
	private final List<Enrollment> enrollments;

	@SuppressWarnings("synthetic-access")
	private UpdateEnrollmentsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.enrollments == null || builder.enrollments.size() == 0)
			throw new IllegalArgumentException("Expected enrollments to not be null.");

		for(Enrollment enrollment : builder.enrollments) {
			if(enrollment == null) throw new IllegalArgumentException("Expected enrollment to not be null.");
			else if(enrollment.getEnrollmentId() == null || (enrollment.getData() == null
				&& enrollment.getEndDate() == null && enrollment.getEntityId() == null && enrollment.getFlags() == null
				&& enrollment.getStatus() == null && enrollment.getStartDate() == null
				&& enrollment.getReference() == null && enrollment.getSchema() == null))

				throw new IllegalArgumentException("Expected enrollmentId and data, endDate, entityId, flags, status, startDate, reference, or schema to not be null.");
		}

		enrollments = Collections.unmodifiableList(new ArrayList<Enrollment>(builder.enrollments));
	}

	public List<Enrollment> getEnrollments() {
		return enrollments;
	}

	/** A builder of immutable UpdateEnrollmentsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Enrollment> enrollments;

		public Builder(final List<Enrollment> enrollments) {
			super();

			if(enrollments == null) this.enrollments = Collections.synchronizedList(new ArrayList<Enrollment>());
			else this.enrollments = Collections.synchronizedList(new ArrayList<Enrollment>(enrollments));
		}

		public Builder(final Enrollment enrollment) {
			super();

			if(enrollment == null) enrollments = Collections.synchronizedList(new ArrayList<Enrollment>());
			else enrollments = Collections.synchronizedList(new ArrayList<Enrollment>(Arrays.asList(enrollment)));
		}

		public Builder addEnrollment(final Enrollment enrollment) {
			enrollments.add(enrollment);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdateEnrollmentsCommand build() {
			return new UpdateEnrollmentsCommand(this);
		}
	}
}