package org.hashtree.jbrainhoney.dlap.element;

import nu.xom.Attribute;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.validate.EmailAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.UsernameAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.UserspaceAttributeValidator;

/** An agent. */
public final class Agent implements XmlElement, Person, Comparable<Agent> {
	private static final long serialVersionUID = -2515055672050817170L;
	private final String domainId;
	private final String domainName;
	private final String email;
	private final String firstName;
	private final String lastName;
	private final String userId;
	private final String username;
	private final String userspace;

	@SuppressWarnings("synthetic-access")
	private Agent(final Builder builder) throws IllegalArgumentException {
		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.email != null && !EmailAttributeValidator.getInstance().isValid(builder.email))
			throw new IllegalArgumentException("Expected email to be valid.");
		else if(builder.firstName != null && builder.firstName.length() > 256)
			throw new IllegalArgumentException("Expected firstName to be less than or equal to 256 characters.");
		else if(builder.lastName != null && builder.lastName.length() > 256)
			throw new IllegalArgumentException("Expected lastName to be less than or equal to 256 characters.");
		else if(builder.userId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.userId))
			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(builder.username != null && !UsernameAttributeValidator.getInstance().isValid(builder.username))
			throw new IllegalArgumentException("Expected username to be valid.");
		else if(builder.userspace != null && !UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");

		domainId = builder.domainId;
		domainName = builder.domainName;
		email = builder.email;
		firstName = builder.firstName;
		lastName = builder.lastName;
		userId = builder.userId;
		username = builder.username;
		userspace = builder.userspace;
	}

	public Agent(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("agent"))
			throw new IllegalArgumentException("Expected agent XML element.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");

		else if(element.getAttribute("email") != null && element.getAttribute("email").getValue().length() > 0
			&& !EmailAttributeValidator.getInstance().isValid(element.getAttribute("email").getValue()))

			throw new IllegalArgumentException("Expected email to be valid.");
		else if(element.getAttribute("firstname") != null && element.getAttribute("firstname").getValue().length() > 256)
			throw new IllegalArgumentException("Expected firstName to be less than or equal to 256 characters.");
		else if(element.getAttribute("lastname") != null && element.getAttribute("lastname").getValue().length() > 256)
			throw new IllegalArgumentException("Expected lastName to be less than or equal to 256 characters.");
		else if(element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("userid").getValue()))

			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(element.getAttribute("username") != null && element.getAttribute("username").getValue().length() > 0
			&& !UsernameAttributeValidator.getInstance().isValid(element.getAttribute("username").getValue()))

			throw new IllegalArgumentException("Expected username to be valid.");
		else if(element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0
			&& !UserspaceAttributeValidator.getInstance().isValid(element.getAttribute("userspace").getValue()))

			throw new IllegalArgumentException("Expected userspace to be valid.");

		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		domainName = (element.getAttribute("domainname") != null && element.getAttribute("domainname").getValue().length() > 0 ?
			element.getAttribute("domainname").getValue() : null);
		email = (element.getAttribute("email") != null && element.getAttribute("email").getValue().length() > 0 ?
			element.getAttribute("email").getValue() : null);
		firstName = (element.getAttribute("firstname") != null && element.getAttribute("firstname").getValue().length() > 0 ?
			element.getAttribute("firstname").getValue() : null);
		lastName = (element.getAttribute("lastname") != null && element.getAttribute("lastname").getValue().length() > 0 ?
			element.getAttribute("lastname").getValue() : null);

		// Referenced as userid and id.
		if(element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0)
			userId = element.getAttribute("userid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			userId = element.getAttribute("id").getValue();
		else userId = null;

		username = (element.getAttribute("username") != null && element.getAttribute("username").getValue().length() > 0 ?
			element.getAttribute("username").getValue() : null);
		userspace = (element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0 ?
			element.getAttribute("userspace").getValue() : null);
	}

	@Override
	public int compareTo(final Agent agent) throws NullPointerException {
		if(agent == null) throw new NullPointerException("Expected agent to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.userId, agent.getUserId());
	}

	public String getDomainId() {
		return domainId;
	}

	public String getDomainName() {
		return domainName;
	}

	public String getEmail() {
		return email;
	}

	public String getFirstName() {
		return firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public String getUserId() {
		return userId;
	}

	public String getUsername() {
		return username;
	}

	public String getUserspace() {
		return userspace;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Agent)) return false;
		else return toString().equals(((Agent)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Agent.class.getSimpleName() + "(");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(domainName != null) stringBuilder.append("domainName=" + domainName + ", ");
		if(email != null) stringBuilder.append("email=" + email + ", ");
		if(firstName != null) stringBuilder.append("firstName=" + firstName + ", ");
		if(lastName != null) stringBuilder.append("lastName=" + lastName + ", ");
		if(userId != null) stringBuilder.append("userId=" + userId + ", ");
		if(username != null) stringBuilder.append("username=" + username + ", ");
		if(userspace != null) stringBuilder.append("userspace=" + userspace + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("agent");

		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(domainName != null) element.addAttribute(new Attribute("domainname", domainName));
		if(email != null) element.addAttribute(new Attribute("email", email));
		if(firstName != null) element.addAttribute(new Attribute("firstname", firstName));
		if(lastName != null) element.addAttribute(new Attribute("lastname", lastName));
		if(userId != null) element.addAttribute(new Attribute("userid", userId));
		if(username != null) element.addAttribute(new Attribute("username", username));
		if(userspace != null) element.addAttribute(new Attribute("userspace", userspace));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Agent elements. */
	public final static class Builder implements XmlElement.Builder {
		private String domainId;
		private String domainName;
		private String email;
		private String firstName;
		private String lastName;
		private String userId;
		private String username;
		private String userspace;

		public Builder() {

		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setDomainName(final String domainName) {
			this.domainName = domainName;

			return this;
		}

		public Builder setEmail(final String email) {
			this.email = email;

			return this;
		}

		public Builder setFirstName(final String firstName) {
			this.firstName = firstName;

			return this;
		}

		public Builder setLastName(final String lastName) {
			this.lastName = lastName;

			return this;
		}

		public Builder setUserId(final String userId) {
			this.userId = userId;

			return this;
		}

		public Builder setUserId(final Integer userId) {
			this.userId = (userId == null ? null : userId.toString());

			return this;
		}

		public Builder setUsername(final String username) {
			this.username = username;

			return this;
		}

		public Builder setUserspace(final String userspace) {
			this.userspace = userspace;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Agent build() {
			return new Agent(this);
		}
	}
}