package org.hashtree.jbrainhoney.dlap.element;

import java.util.Date;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.enumeration.DateBoundary;
import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.GuidAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.ReferenceAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.UserspaceAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.VersionAttributeValidator;
import org.hashtree.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** A domain. */
public final class Domain implements XmlElement, Entity, Partition, Comparable<Domain> {
	private static final long serialVersionUID = 903421567205366543L;
	private final DateTime creationDate;
	private final Element data;
	private final String domainId;
	private final String guid;
	private final String name;
	private final String parentId;
	private final String reference;
	private final String userspace;
	private final Integer version;

	@SuppressWarnings("synthetic-access")
	private Domain(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.name != null && builder.name.length() > 128)
			throw new IllegalArgumentException("Expected name to be less than or equal to 128 characters.");
		else if(builder.parentId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.parentId))
			throw new IllegalArgumentException("Expected parentId to be valid.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.userspace != null && !UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");
		else if(builder.version != null && !VersionAttributeValidator.getInstance().isValid(builder.version))
			throw new IllegalArgumentException("Expected version to be valid.");

		creationDate = builder.creationDate;
		data = xmlElement;
		domainId = builder.domainId;
		guid = builder.guid;
		name = builder.name;
		parentId = builder.parentId;
		reference = builder.reference;
		userspace = builder.userspace;
		version = builder.version;
	}

	public Domain(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("domain"))
			throw new IllegalArgumentException("Expected domain XML element.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("name") != null && element.getAttribute("name").getValue().length() > 128)
			throw new IllegalArgumentException("Expected name to be less than or equal to 128 characters.");
		else if(element.getAttribute("parentid") != null && element.getAttribute("parentid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("parentid").getValue()))

			throw new IllegalArgumentException("Expected parentId to be valid.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0
			&& !UserspaceAttributeValidator.getInstance().isValid(element.getAttribute("userspace").getValue()))

			throw new IllegalArgumentException("Expected userspace to be valid.");
		else if(element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0
			&& !VersionAttributeValidator.getInstance().isValid(element.getAttribute("version").getValue()))

			throw new IllegalArgumentException("Expected parentId to be valid.");

		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);

		// Referenced as domainid and id.
		if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0)
			domainId = element.getAttribute("domainid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			domainId = element.getAttribute("id").getValue();
		else domainId = null;

		guid = (element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0 ?
			element.getAttribute("guid").getValue() : null);
		name = (element.getAttribute("name") != null && element.getAttribute("name").getValue().length() > 0 ?
			element.getAttribute("name").getValue() : null);
		parentId = (element.getAttribute("parentid") != null && element.getAttribute("parentid").getValue().length() > 0 ?
			element.getAttribute("parentid").getValue() : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);
		userspace = (element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0 ?
			element.getAttribute("userspace").getValue() : null);
		version = (element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("version").getValue()) : null);
	}

	@Override
	public int compareTo(final Domain domain) throws NullPointerException {
		if(domain == null) throw new NullPointerException("Expected domain to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.domainId, domain.getDomainId());
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public String getDomainId() {
		return domainId;
	}

	public String getGuid() {
		return guid;
	}

	public String getName() {
		return name;
	}

	public String getParentId() {
		return parentId;
	}

	public String getReference() {
		return reference;
	}

	public String getUserspace() {
		return userspace;
	}

	public Integer getVersion() {
		return version;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Domain)) return false;
		else return toString().equals(((Domain)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Domain.class.getSimpleName() + "(");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(name != null) stringBuilder.append("name=" + name + ", ");
		if(parentId != null) stringBuilder.append("parentId=" + parentId + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(userspace != null) stringBuilder.append("userspace=" + userspace + ", ");
		if(version != null) stringBuilder.append("version=" + version.toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("domain");

		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(name != null) element.addAttribute(new Attribute("name", name));
		if(parentId != null) element.addAttribute(new Attribute("parentid", parentId));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(userspace != null) element.addAttribute(new Attribute("userspace", userspace));
		if(version != null) element.addAttribute(new Attribute("version", version.toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Domain elements. */
	public final static class Builder implements XmlElement.Builder {
		private DateTime creationDate;
		private String data;
		private String domainId;
		private String guid;
		private String name;
		private String parentId;
		private String reference;
		private String userspace;
		private Integer version;

		public Builder() {

		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setName(final String name) {
			this.name = name;

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setParentId(final String parentId) {
			this.parentId = parentId;

			return this;
		}

		public Builder setParentId(final Integer parentId) {
			this.parentId = (parentId == null ? null : parentId.toString());

			return this;
		}

		public Builder setUserspace(final String userspace) {
			this.userspace = userspace;

			return this;
		}

		public Builder setVersion(final Integer version) {
			this.version = version;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Domain build() {
			return new Domain(this);
		}
	}
}