package org.hashtree.jbrainhoney.dlap.element;

import java.util.Date;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.enumeration.DateBoundary;
import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.GuidAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.ReferenceAttributeValidator;
import org.hashtree.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** A group. */
public final class Group implements XmlElement, Entity, Partition, Comparable<Group> {
	private static final long serialVersionUID = -9823567165908330L;
	private final DateTime creationDate;
	private final Element data;
	private final String domainId;
	private final String guid;
	private final String groupId;
	private final String ownerId;
	private final String reference;
	private final Integer setId;
	private final String title;

	@SuppressWarnings("synthetic-access")
	private Group(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.groupId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.groupId))
			throw new IllegalArgumentException("Expected groupId to be valid.");
		else if(builder.ownerId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.ownerId))
			throw new IllegalArgumentException("Expected ownerId to be valid.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.setId != null && builder.setId <= 0)
			throw new IllegalArgumentException("Expected setId to be greater than 0.");
		else if(builder.title != null && builder.title.length() > 128)
			throw new IllegalArgumentException("Expected title to be less than or equal to 128 characters.");

		creationDate = builder.creationDate;
		data = xmlElement;
		domainId = builder.domainId;
		guid = builder.guid;
		groupId = builder.groupId;
		ownerId = builder.ownerId;
		reference = builder.reference;
		setId = builder.setId;
		title = builder.title;
	}

	public Group(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("group"))
			throw new IllegalArgumentException("Expected group XML element.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("groupid") != null && element.getAttribute("groupid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("groupid").getValue()))

			throw new IllegalArgumentException("Expected groupId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected groupId to be valid.");
		else if(element.getAttribute("ownerid") != null && element.getAttribute("ownerid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("ownerid").getValue()))

			throw new IllegalArgumentException("Expected ownerId to be valid.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("setid") != null && element.getAttribute("setid").getValue().length() > 0
			&& Integer.parseInt(element.getAttribute("setid").getValue()) <= 0)

			throw new IllegalArgumentException("Expected setId to be greater than 0.");
		else if(element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 128)
			throw new IllegalArgumentException("Expected title to be less than or equal to 128 characters.");

		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);
		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		guid = (element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0 ?
			element.getAttribute("guid").getValue() : null);

		// Referenced as groupid and id.
		if(element.getAttribute("groupid") != null && element.getAttribute("groupid").getValue().length() > 0)
			groupId = element.getAttribute("groupid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			groupId = element.getAttribute("id").getValue();
		else groupId = null;

		ownerId = (element.getAttribute("ownerid") != null && element.getAttribute("ownerid").getValue().length() > 0 ?
			element.getAttribute("ownerid").getValue() : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);
		setId = (element.getAttribute("setid") != null && element.getAttribute("setid").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("setid").getValue()) : null);
		title = (element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 0 ?
			element.getAttribute("title").getValue() : null);
	}

	@Override
	public int compareTo(final Group group) throws NullPointerException {
		if(group == null) throw new NullPointerException("Expected group to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.groupId, group.getGroupId());
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public String getDomainId() {
		return domainId;
	}

	public String getGuid() {
		return guid;
	}

	public String getGroupId() {
		return groupId;
	}

	public String getOwnerId() {
		return ownerId;
	}

	public String getReference() {
		return reference;
	}

	public Integer getSetId() {
		return setId;
	}

	public String getTitle() {
		return title;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Group)) return false;

		return toString().equals(((Group)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Group.class.getSimpleName() + "(");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(groupId != null) stringBuilder.append("groupId=" + groupId + ", ");
		if(ownerId != null) stringBuilder.append("ownerId=" + ownerId + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(setId != null) stringBuilder.append("setId=" + setId.toString() + ", ");
		if(title != null) stringBuilder.append("title=" + title + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("group");

		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(groupId != null) element.addAttribute(new Attribute("groupid", groupId));
		if(ownerId != null) element.addAttribute(new Attribute("ownerid", ownerId));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(setId != null) element.addAttribute(new Attribute("setid", setId.toString()));
		if(title != null) element.addAttribute(new Attribute("title", title));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Group elements. */
	public final static class Builder implements XmlElement.Builder {
		private DateTime creationDate;
		private String data;
		private String domainId;
		private String guid;
		private String groupId;
		private String ownerId;
		private String reference;
		private Integer setId;
		private String title;

		public Builder() {

		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setGroupId(final String groupId) {
			this.groupId = groupId;

			return this;
		}

		public Builder setGroupId(final Integer groupId) {
			this.groupId = (groupId == null ? null : groupId.toString());

			return this;
		}

		public Builder setOwnerId(final String ownerId) {
			this.ownerId = ownerId;

			return this;
		}

		public Builder setOwnerId(final Integer ownerId) {
			this.ownerId = (ownerId == null ? null : ownerId.toString());

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setSetId(final Integer setId) {
			this.setId = setId;

			return this;
		}

		public Builder setTitle(final String title) {
			this.title = title;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Group build() {
			return new Group(this);
		}
	}
}