package org.hashtree.jbrainhoney.dlap.element;

import java.util.Date;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.enumeration.DateBoundary;
import org.hashtree.jbrainhoney.dlap.enumeration.EnrollmentStatus;
import org.hashtree.jbrainhoney.dlap.enumeration.SectionType;
import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.GuidAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.ReferenceAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.VersionAttributeValidator;
import org.hashtree.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** A section. */
public final class Section implements XmlElement, Class, Entity, Partition, Comparable<Section> {
	private static final long serialVersionUID = 7682955409123526544L;
	private final String courseId;
	private final DateTime creationDate;
	private final Element data;
	private final Integer days;
	private final String domainId;
	private final DateTime endDate;
	private final String guid;
	private final String reference;
	private final String sectionId;
	private final EnrollmentStatus status;
	private final DateTime startDate;
	private final String term;
	private final String title;
	private final SectionType type;
	private final Integer version;

	@SuppressWarnings("synthetic-access")
	private Section(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.courseId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.courseId))
			throw new IllegalArgumentException("Expected courseId to be valid.");

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.days != null && builder.days < 0)
			throw new IllegalArgumentException("Expected days to be greater than or equal to 0.");
		else if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.sectionId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.sectionId))
			throw new IllegalArgumentException("Expected sectionId to be valid.");
		else if(builder.term != null && builder.term.length() > 128)
			throw new IllegalArgumentException("Expected term to be less than 128 characters.");
		else if(builder.title != null && builder.title.length() > 128)
			throw new IllegalArgumentException("Expected title to be less than 128 characters.");
		else if(builder.version != null && !VersionAttributeValidator.getInstance().isValid(builder.version))
			throw new IllegalArgumentException("Expected version to be valid.");

		courseId = builder.courseId;
		creationDate = builder.creationDate;
		data = xmlElement;
		days = builder.days;
		domainId = builder.domainId;
		endDate = builder.endDate;
		guid = builder.guid;
		reference = builder.reference;
		sectionId = builder.sectionId;
		status = builder.status;
		startDate = builder.startDate;
		term = builder.term;
		title = builder.title;
		type = builder.type;
		version = builder.version;
	}

	public Section(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("section"))
			throw new IllegalArgumentException("Expected section XML element.");
		else if(element.getAttribute("courseid") != null && element.getAttribute("courseid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("courseid").getValue()))

			throw new IllegalArgumentException("Expected courseId to be valid.");
		else if(element.getAttribute("days") != null && element.getAttribute("days").getValue().length() > 0
			&& Integer.parseInt(element.getAttribute("days").getValue()) < 0)

			throw new IllegalArgumentException("Expected days to be greater than or equal to 0.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("sectionid") != null && element.getAttribute("sectionid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("sectionid").getValue()))

			throw new IllegalArgumentException("Expected sectionId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected sectionId to be valid.");
		else if(element.getAttribute("term") != null && element.getAttribute("term").getValue().length() > 128)
			throw new IllegalArgumentException("Expected term to be less than 128 characters.");
		else if(element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 128)
			throw new IllegalArgumentException("Expected title to be less than 128 characters.");
		else if(element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0
			&& !VersionAttributeValidator.getInstance().isValid(element.getAttribute("version").getValue()))

			throw new IllegalArgumentException("Expected version to be valid.");

		courseId = (element.getAttribute("courseid") != null && element.getAttribute("courseid").getValue().length() > 0 ?
			element.getAttribute("courseid").getValue() : null);
		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);
		days = (element.getAttribute("days") != null && element.getAttribute("days").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("days").getValue()) : null);
		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		endDate = (element.getAttribute("enddate") != null && element.getAttribute("enddate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("enddate").getValue()) : null);
		guid = (element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0 ?
			element.getAttribute("guid").getValue() : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);

		// Referenced as sectionid and id.
		if(element.getAttribute("sectionid") != null && element.getAttribute("sectionid").getValue().length() > 0)
			sectionId = element.getAttribute("sectionid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			sectionId = element.getAttribute("id").getValue();
		else sectionId = null;

		status = (element.getAttribute("status") != null && element.getAttribute("status").getValue().length() > 0 ?
			EnrollmentStatus.valueOfValue(Integer.parseInt(element.getAttribute("status").getValue())) : null);
		startDate = (element.getAttribute("startdate") != null && element.getAttribute("startdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("startdate").getValue()) : null);
		term = (element.getAttribute("term") != null && element.getAttribute("term").getValue().length() > 0 ?
			element.getAttribute("term").getValue() : null);
		title = (element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 0 ?
			element.getAttribute("title").getValue() : null);
		type = (element.getAttribute("type") != null && element.getAttribute("type").getValue().length() > 0 ?
			SectionType.valueOfValue(element.getAttribute("type").getValue()) : null);
		version = (element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("version").getValue()) : null);
	}

	@Override
	public int compareTo(final Section section) throws NullPointerException {
		if(section == null) throw new NullPointerException("Expected section to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.sectionId, section.getSectionId());
	}

	public String getCourseId() {
		return courseId;
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public Integer getDays() {
		return days;
	}

	public String getDomainId() {
		return domainId;
	}

	public DateTime getEndDate() {
		return endDate;
	}

	public String getGuid() {
		return guid;
	}

	public String getReference() {
		return reference;
	}

	public String getSectionId() {
		return sectionId;
	}

	public EnrollmentStatus getStatus() {
		return status;
	}

	public DateTime getStartDate() {
		return startDate;
	}

	public String getTerm() {
		return term;
	}

	public String getTitle() {
		return title;
	}

	public SectionType getType() {
		return type;
	}

	public Integer getVersion() {
		return version;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Section)) return false;
		else return toString().equals(((Section)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Section.class.getSimpleName() + "(");
		if(courseId != null) stringBuilder.append("courseId=" + courseId + ", ");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(days != null) stringBuilder.append("days=" + days.toString() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(endDate != null) stringBuilder.append("endDate=" + endDate.toString() + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(sectionId != null) stringBuilder.append("sectionId=" + sectionId + ", ");
		if(status != null) stringBuilder.append("status=" + status.getValue().toString() + ", ");
		if(startDate != null) stringBuilder.append("startDate=" + startDate.toString() + ", ");
		if(term != null) stringBuilder.append("term=" + term + ", ");
		if(title != null) stringBuilder.append("title=" + title + ", ");
		if(type != null) stringBuilder.append("type=" + type.getValue().toString() + ", ");
		if(version != null) stringBuilder.append("version=" + version.toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("section");

		if(courseId != null) element.addAttribute(new Attribute("courseid", courseId));
		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(days != null) element.addAttribute(new Attribute("days", days.toString()));
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(endDate != null) element.addAttribute(new Attribute("enddate", endDate.toString()));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(sectionId != null) element.addAttribute(new Attribute("sectionid", sectionId));
		if(status != null) element.addAttribute(new Attribute("status", status.getValue().toString()));
		if(startDate != null) element.addAttribute(new Attribute("startDate", startDate.toString()));
		if(term != null) element.addAttribute(new Attribute("term", term));
		if(title != null) element.addAttribute(new Attribute("title", title));
		if(type != null) element.addAttribute(new Attribute("type", type.getValue().toString()));
		if(version != null) element.addAttribute(new Attribute("version", version.toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Section elements. */
	public final static class Builder implements XmlElement.Builder {
		private String courseId;
		private DateTime creationDate;
		private String data;
		private Integer days;
		private String domainId;
		private DateTime endDate;
		private String guid;
		private String reference;
		private String sectionId;
		private EnrollmentStatus status;
		private DateTime startDate;
		private String term;
		private String title;
		private SectionType type;
		private Integer version;

		public Builder() {

		}

		public Builder setCourseId(final String courseId) {
			this.courseId = courseId;

			return this;
		}

		public Builder setCourseId(final Integer courseId) {
			this.courseId = (courseId == null ? null : courseId.toString());

			return this;
		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDays(final Integer days) {
			this.days = days;

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setEndDate(final Date endDate) {
			this.endDate = (endDate == null ? null : new DateTime(endDate));

			return this;
		}

		public Builder setEndDate(final DateTime endDate) {
			this.endDate = endDate;

			return this;
		}

		public Builder setEndDate(final DateBoundary endDate) {
			this.endDate = endDate.getValue();

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setSectionId(final String sectionId) {
			this.sectionId = sectionId;

			return this;
		}

		public Builder setSectionId(final Integer sectionId) {
			this.sectionId = (sectionId == null ? null : sectionId.toString());

			return this;
		}

		public Builder setStatus(final EnrollmentStatus status) {
			this.status = status;

			return this;
		}

		public Builder setStartDate(final Date startDate) {
			this.startDate = (startDate == null ? null : new DateTime(startDate));

			return this;
		}

		public Builder setStartDate(final DateTime startDate) {
			this.startDate = startDate;

			return this;
		}

		public Builder setStartDate(final DateBoundary startDate) {
			this.startDate = startDate.getValue();

			return this;
		}

		public Builder setTerm(final String term) {
			this.term = term;

			return this;
		}

		public Builder setTitle(final String title) {
			this.title = title;

			return this;
		}

		public Builder setType(final SectionType type) {
			this.type = type;

			return this;
		}

		public Builder setType(final String type) {
			this.type = (type == null ? null : SectionType.valueOfValue(type));

			return this;
		}

		public Builder setVersion(final Integer version) {
			this.version = version;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Section build() {
			return new Section(this);
		}
	}
}